(*:Date: Tue Feb  3 04:40:04 EST 1998
*)

(*:Title: GEO.m *)

(*:Author: Alfred Gray *)

(* Copyright 1996-1998 by Alfred Gray *)

(*:Package Version: 1.0 *)

(*:Mathematica Version: 2.2, 3.0 *)

(*:Summary: 
This package consists of formulas for the geodesics of many parametrized surfaces.*)

BeginPackage["GEO`","SURFS`","CSPROGS`"]

Begin["`Private`"]

     geo::usage="{u,v}->geo[x][p,q,u,v] gives the system of first order differential equations for a patch x:U->R^n. geo[ee,ff,gg][p,q,u,v] gives the system of first order differential equations for the geodesics of a metric with coefficients ee,ff,gg."

geo[catalan][p_,q_,u_,v_]:=
	{p,q,
	-(q^2*Sin[u])/(2*(Cos[u] - Cosh[v])) -
	(p^2*Sin[u])/(2*(-Cos[u] + Cosh[v])) -
	(p*q*(1 - Cos[u] + 2*Cosh[v])*Tanh[v/2])/
	(-Cos[u] + Cosh[v]),
	-((p*q*Sin[u])/
	(-Cos[u] + Cosh[v])) - (p^2*(1 - Cos[u] +
	2*Cosh[v])*Tanh[v/2])/(2*(Cos[u] - Cosh[v])) -
	(q^2*(1 - Cos[u] + 2*Cosh[v])*Tanh[v/2])/
	(2*(-Cos[u] + Cosh[v]))}

geo[catenoid[a_,b_,c_]][p_,q_,u_,v_]:=
	{p,q,
	 -(p^2*Cosh[v]^2*(a^2 - b^2)*c^2*Sin[2*u])/
	(2*((Cos[u]^2*Cosh[v]^2*b^2 + Cosh[v]^2*a^2*Sin[u]^2)*
	(c^2 + Cos[u]^2*a^2*Sinh[v]^2 + b^2*Sin[u]^2*Sinh[v]^2) -
	((a^2 - b^2)^2*Sin[2*u]^2*Sinh[2*v]^2)/16)) -
	(q^2*Cosh[v]^2*(-a^2 + b^2)*c^2*Sin[2*u])/
	(2*((Cos[u]^2*Cosh[v]^2*b^2 + Cosh[v]^2*a^2*Sin[u]^2)*
	(c^2 + Cos[u]^2*a^2*Sinh[v]^2 + b^2*Sin[u]^2*Sinh[v]^2) -
	 ((a^2 - b^2)^2*Sin[2*u]^2*Sinh[2*v]^2)/16)) - 2*p*q*Tanh[v],
	(p^2*Cosh[v]^3*a^2*b^2*Sinh[v])/((Cos[u]^2*Cosh[v]^2*b^2 +
	Cosh[v]^2*a^2*Sin[u]^2)*(c^2 + Cos[u]^2*a^2*Sinh[v]^2 +
	b^2*Sin[u]^2*Sinh[v]^2) -
	((a^2 - b^2)^2*Sin[2*u]^2*Sinh[2*v]^2)/16) -
	(q^2*Cosh[v]^3*a^2*b^2*Sinh[v])/
	((Cos[u]^2*Cosh[v]^2*b^2 + Cosh[v]^2*a^2*Sin[u]^2)*
	(c^2 + Cos[u]^2*a^2*Sinh[v]^2 + b^2*Sin[u]^2*Sinh[v]^2) -
	((a^2 - b^2)^2*Sin[2*u]^2*Sinh[2*v]^2)/16)}

geo[dini[a_,b_]][p_,q_,u_,v_]:=
	{p,q,
	-((p^2*Cos[v]*a*b)/(a^2 + b^2)) -
	(p*q*Cos[v]*Csc[v/2]*a^2*Sec[v/2])/(a^2 + b^2) +
	(q^2*Cos[v]*Csc[v/2]^2*a*b*Sec[v/2]^2)/(4*(a^2 + b^2)),
	(2*p*q*Cos[v]*a*b)/(a^2 + b^2) - (q^2*Csc[v/2]*
	(-2*a^2 - b^2 + Cos[2*v]*b^2)*Sec[v/2]*Sec[v])/(4*(a^2 + b^2)) -
	(p^2*(-a^2 + Cos[2*v]*a^2 - 2*b^2)*Tan[v])/(2*(a^2 + b^2))}

geo[eightsurface][p_,q_,u_,v_]:=
	{p,q,
	-4*p*q*Cot[2*v],
	(2*p^2*Sin[4*v])/(5 + Cos[2*v] +4*Cos[4*v]) +
	(q^2*(Sin[2*v] + 8*Sin[4*v]))/(5 + Cos[2*v] + 4*Cos[4*v])}

geo[ellipsoid[a_,b_,c_]][p_,q_,u_,v_]:=
	{p,q,
	 -(q^2*Cos[v]^2*(a^2 - b^2)*c^2*Sin[2*u])/
	(2*((Cos[u]^2*Cos[v]^2*b^2 + Cos[v]^2*a^2*Sin[u]^2)*
	(Cos[v]^2*c^2 + Cos[u]^2*a^2*Sin[v]^2 + b^2*Sin[u]^2*Sin[v]^2) -
	((a^2 - b^2)^2*Sin[2*u]^2*Sin[2*v]^2)/16)) -
	(p^2*Cos[v]^4*(a^2 - b^2)*c^2*Sin[2*u])/
	(2*((Cos[u]^2*Cos[v]^2*b^2 + Cos[v]^2*a^2*Sin[u]^2)*
	(Cos[v]^2*c^2 + Cos[u]^2*a^2*Sin[v]^2 + b^2*Sin[u]^2*Sin[v]^2) -
	((a^2 - b^2)^2*Sin[2*u]^2*Sin[2*v]^2)/16)) + 2*p*q*Tan[v],
	-((p^2*Cos[v]^3*a^2*b^2*Sin[v])/((Cos[u]^2*Cos[v]^2*b^2 +
	Cos[v]^2*a^2*Sin[u]^2)*(Cos[v]^2*c^2 + Cos[u]^2*a^2*Sin[v]^2 +
	b^2*Sin[u]^2*Sin[v]^2) -
	((a^2 - b^2)^2*Sin[2*u]^2*Sin[2*v]^2)/16)) -
	(q^2*Cos[v]^3*(2*a^2*b^2 - a^2*c^2 + Cos[2*u]*a^2*c^2 -
	b^2*c^2 - Cos[2*u]*b^2*c^2)*Sin[v])/
	(2*((Cos[u]^2*Cos[v]^2*b^2 + Cos[v]^2*a^2*Sin[u]^2)*
	(Cos[v]^2*c^2 + Cos[u]^2*a^2*Sin[v]^2 +
	b^2*Sin[u]^2*Sin[v]^2) -
	((a^2 - b^2)^2*Sin[2*u]^2*Sin[2*v]^2)/16))}

geo[enneper][p_,q_,u_,v_]:=
	{p,q,
	(-2*p^2*u)/(1 + u^2 + v^2) + (2*q^2*u)/(1 + u^2 + v^2) -
	 (4*p*q*v)/(1 + u^2 + v^2),
	(-4*p*q*u)/(1 + u^2 + v^2) + (2*p^2*v)/(1 + u^2 + v^2) -
	(2*q^2*v)/(1 + u^2 + v^2)}

geo[funnel[a_,b_,c_]][p_,q_,u_,v_]:=
	{p,q,
	(-2*p*q)/v - (p^2*(a^2 - b^2)*c^2*Sin[2*u])/
	(2*v^2*a^2*b^2 + a^2*c^2 - Cos[2*u]*a^2*c^2 + b^2*c^2 +
	Cos[2*u]*b^2*c^2) - (q^2*(-a^2 + b^2)*
	c^2*Sin[2*u])/(2*v^2*((v^2*Cos[u]^2*b^2 + v^2*a^2*Sin[u]^2)*
	(Cos[u]^2*a^2 + c^2/v^2 + b^2*Sin[u]^2) -
	(v^2*(a^2 - b^2)^2*Sin[2*u]^2)/4)),
	(p^2*v^3*a^2*b^2)/
	((v^2*Cos[u]^2*b^2 + v^2*a^2*Sin[u]^2)*
	(Cos[u]^2*a^2 + c^2/v^2 + b^2*Sin[u]^2) -
	(v^2*(a^2 - b^2)^2*Sin[2*u]^2)/4) +
	(q^2*c^2*(v^2*Cos[u]^2*b^2 + v^2*a^2*Sin[u]^2))/
	(v^3*((v^2*Cos[u]^2*b^2 + v^2*a^2*Sin[u]^2)*
	(Cos[u]^2*a^2 + c^2/v^2 + b^2*Sin[u]^2) -
	(v^2*(a^2 - b^2)^2*Sin[2*u]^2)/4))}

geo[handkerchief[a_]][p_,q_,u_,v_]:=
	{p,q,
	(-4*p*q*v*(u^2 + v^2 + 2*u*a))/
	(1 + u^4 + 6*u^2*v^2 + v^4 + 4*u^3*a - 4*u*v^2*a +
	4*u^2*a^2 + 4*v^2*a^2) - (2*q^2*(u - a)*(u^2 + v^2 + 2*u*a))/
	(1 + u^4 + 6*u^2*v^2 + v^4 + 4*u^3*a - 4*u*v^2*a + 4*u^2*a^2 +
	4*v^2*a^2) - (2*p^2*(u + a)*(u^2 + v^2 + 2*u*a))/
	(1 + u^4 + 6*u^2*v^2 + v^4 + 4*u^3*a -
	4*u*v^2*a + 4*u^2*a^2 + 4*v^2*a^2),
	(-8*p*q*v^2*(u - a))/(1 + u^4 + 6*u^2*v^2 + v^4 +
	4*u^3*a - 4*u*v^2*a + 4*u^2*a^2 + 4*v^2*a^2) -
	(4*q^2*v*(-u + a)^2)/(1 + u^4 + 6*u^2*v^2 + v^4 +
	4*u^3*a - 4*u*v^2*a + 4*u^2*a^2 + 4*v^2*a^2) -
	(4*p^2*v*(u^2 - a^2))/(1 + u^4 + 6*u^2*v^2 + v^4 +
	4*u^3*a - 4*u*v^2*a + 4*u^2*a^2 + 4*v^2*a^2)}

geo[helicoid[a_,b_,c_]][p_,q_,u_,v_]:=
	{p,q,
         (-2*p*q*v*a^2*b^2)/
	((v^2*Cos[u]^2*b^2 + c^2 + v^2*a^2*Sin[u]^2)*
	(Cos[u]^2*a^2 + b^2*Sin[u]^2) -
	(v^2*(a^2 - b^2)^2*Sin[2*u]^2)/4),p^2*v +
	 (p*q*(a^2 - b^2)*c^2*Sin[2*u])/
	((v^2*Cos[u]^2*b^2 + c^2 + v^2*a^2*Sin[u]^2)*
	(Cos[u]^2*a^2 + b^2*Sin[u]^2) - (v^2*(a^2 - b^2)^2*Sin[2*u]^2)/4)}

geo[heltorus[a_,b_,c_]][p_,q_,u_,v_]:=
	{p,q,
	(4*p*q*b*(a + Cos[v]*b)*Sin[v])/
	(2*a^2 + 4*Cos[v]*a*b + b^2 + Cos[2*v]*b^2 + c^2 -
	Cos[2*v]*c^2) +
	(2*q^2*b*c*Sin[v])/(2*a^2 + 4*Cos[v]*a*b + b^2 +
	Cos[2*v]*b^2 + c^2 - Cos[2*v]*c^2) -
	(p^2*(a + Cos[v]*b)*c*Sin[2*v])/
	(-2*a^2 - 4*Cos[v]*a*b - b^2 - Cos[2*v]*b^2 -
	c^2 + Cos[2*v]*c^2),
	-((p^2*(a + Cos[v]*b)*(2*a^2 + 4*Cos[v]*a*b + b^2 +
	Cos[2*v]*b^2 + 2*c^2)*Sin[v])/
	(b*(2*a^2 + 4*Cos[v]*a*b + b^2 +
	Cos[2*v]*b^2 + c^2 - Cos[2*v]*c^2))) -
	(2*p*q*(a + Cos[v]*b)*c*Sin[2*v])/
	(2*a^2 + 4*Cos[v]*a*b + b^2 + Cos[2*v]*b^2 + c^2 -
	Cos[2*v]*c^2) - (q^2*c^2*Sin[2*v])/
	(2*a^2 + 4*Cos[v]*a*b + b^2 + Cos[2*v]*b^2 + c^2 - Cos[2*v]*c^2)}

geo[hyperbolicparaboloid][p_,q_,u_,v_]:=
	{p,q,
	(-2*p*q*v)/(1 + u^2 + v^2),
	(-2*p*q*u)/(1 + u^2 + v^2)}

geo[hyperboloid[a_,b_,c_]][p_,q_,u_,v_]:=
	{p,q,
	-(p^2*Cosh[v]^4*(a^2 - b^2)*c^2*Sin[2*u])/
	(2*((Cos[u]^2*Cosh[v]^2*b^2 +
	 Cosh[v]^2*a^2*Sin[u]^2)*(Cosh[v]^2*c^2 +
	 Cos[u]^2*a^2*Sinh[v]^2 + b^2*Sin[u]^2*Sinh[v]^2) -
	 ((a^2 - b^2)^2*Sin[2*u]^2*Sinh[2*v]^2)/16)) -
	 (q^2*Cosh[v]^2*(-a^2 + b^2)*c^2*Sin[2*u])/
	(2*((Cos[u]^2*Cosh[v]^2*b^2 + Cosh[v]^2*a^2*Sin[u]^2)*
	(Cosh[v]^2*c^2 + Cos[u]^2*a^2*Sinh[v]^2 + b^2*Sin[u]^2*Sinh[v]^2) -
	((a^2 - b^2)^2*Sin[2*u]^2*Sinh[2*v]^2)/16)) - 2*p*q*Tanh[v],
	(p^2*Cosh[v]^3*a^2*b^2*Sinh[v])/
	((Cos[u]^2*Cosh[v]^2*b^2 + Cosh[v]^2*a^2*Sin[u]^2)*
	(Cosh[v]^2*c^2 + Cos[u]^2*a^2*Sinh[v]^2 +
	 b^2*Sin[u]^2*Sinh[v]^2) -
	 ((a^2 - b^2)^2*Sin[2*u]^2*Sinh[2*v]^2)/16) -
	 (q^2*Cosh[v]^3*(2*a^2*b^2 + a^2*c^2 -
	 Cos[2*u]*a^2*c^2 + b^2*c^2 +
         Cos[2*u]*b^2*c^2)*Sinh[v])/(2*((Cos[u]^2*Cosh[v]^2*b^2 +
	Cosh[v]^2*a^2*Sin[u]^2)*(Cosh[v]^2*c^2 +
	 Cos[u]^2*a^2*Sinh[v]^2 + b^2*Sin[u]^2*Sinh[v]^2) -
	 ((a^2 - b^2)^2*Sin[2*u]^2*Sinh[2*v]^2)/16))}

geo[hy2sheet[a_,b_,c_]][p_,q_,u_,v_]:=
	{p,q,
	-(q^2*Cosh[v]^2*(a^2 + b^2)*c^2*Sinh[2*u])/
	(2*((Cosh[u]^2*Cosh[v]^2*b^2 + Cosh[v]^2*a^2*Sinh[u]^2)*
	(Cosh[v]^2*c^2 + Cosh[u]^2*a^2*Sinh[v]^2 + b^2*Sinh[u]^2*Sinh[v]^2) -
	((a^2 +b^2)^2*Sinh[2*u]^2*Sinh[2*v]^2)/16)) -
	(p^2*Cosh[v]^4*(a^2 + b^2)*c^2*Sinh[2*u])/
	(2*((Cosh[u]^2*Cosh[v]^2*b^2 + Cosh[v]^2*a^2*Sinh[u]^2)*
	(Cosh[v]^2*c^2 + Cosh[u]^2*a^2*Sinh[v]^2 + b^2*Sinh[u]^2*Sinh[v]^2) -
	((a^2 + b^2)^2*Sinh[2*u]^2*Sinh[2*v]^2)/16)) -	2*p*q*Tanh[v],
	-((p^2*Cosh[v]^3*a^2*b^2*Sinh[v])/
	((Cosh[u]^2*Cosh[v]^2*b^2 + Cosh[v]^2*a^2*Sinh[u]^2)*
	(Cosh[v]^2*c^2 + Cosh[u]^2*a^2*Sinh[v]^2 + b^2*Sinh[u]^2*Sinh[v]^2) -
	((a^2 + b^2)^2*Sinh[2*u]^2*Sinh[2*v]^2)/16)) -
	(q^2*Cosh[v]^3*(2*a^2*b^2 - a^2*c^2 +
	Cosh[2*u]*a^2*c^2 + b^2*c^2 + Cosh[2*u]*b^2*c^2)*Sinh[v])/
	(2*((Cosh[u]^2*Cosh[v]^2*b^2 + Cosh[v]^2*a^2*Sinh[u]^2)*
	(Cosh[v]^2*c^2 + Cosh[u]^2*a^2*Sinh[v]^2 + b^2*Sinh[u]^2*Sinh[v]^2) -
	((a^2 + b^2)^2*Sinh[2*u]^2*Sinh[2*v]^2)/16))}

geo[kuen][p_,q_,u_,v_]:=
	{p,q,
	-((p^2*(2 - u^2 + u^2*Cos[2*v]))/(2*u + u^3 - u^3*Cos[2*v])) -
	(2*p*q*(2 - u^2 + u^2*Cos[2*v])*Cot[v])/(2 + u^2 - u^2*Cos[2*v]) +
	(q^2*(2 - u^2 + u^2*Cos[2*v])*Csc[v]^2)/(-2*u - u^3 + u^3*Cos[2*v]),
	-((q^2*Cos[v]*(-8 - 16*u^2 + 3*u^4 + 16*u^2*Cos[2*v] -
	4*u^4*Cos[2*v] +  u^4*Cos[4*v])*
	Csc[v/2]*Sec[v/2])/(16 - 6*u^4 + 8*u^4*Cos[2*v] - 2*u^4*Cos[4*v])) -
	(256*p*q*u*Cos[v/2]^2*Sin[v/2]^2)/
	(-8 + 3*u^4 - 4*u^4*Cos[2*v] + u^4*Cos[4*v]) -
	(256*p^2*u^2*Cos[v/2]^3*Cos[v]*Sin[v/2]^3)/
	(-8 + 3*u^4 - 4*u^4*Cos[2*v] + u^4*Cos[4*v])}

geo[menn[a_]][p_,q_,u_,v_]:={p,q,(4*q^2*u*(v + 2*u^2*a))/(1 + u^4 - 4*u^2*v + 4*v^2 + 4*u^2*v^2 + 16*u^4*v*a + 16*u^6*a^2) - (8*p*q*u^2*(v + 2*u^2*a))/(1 + u^4 - 4*u^2*v + 4*v^2 + 4*u^2*v^2 + 16*u^4*v*a + 16*u^6*a^2) - (4*p^2*u*(v^2 + 8*u^2*v*a + 12*u^4*a^2))/(1 + u^4 - 4*u^2*v + 4*v^2 + 4*u^2*v^2 + 16*u^4*v*a + 16*u^6*a^2),(-4*p*q*u*(u^2 - 2*v))/(1 + u^4 - 4*u^2*v + 4*v^2 + 4*u^2*v^2 + 16*u^4*v*a + 16*u^6*a^2) - (2*q^2*(-u^2 + 2*v))/(1 + u^4 - 4*u^2*v + 4*v^2 + 4*u^2*v^2 + 16*u^4*v*a + 16*u^6*a^2) - (2*p^2*(u^2 - 2*v)*(v + 6*u^2*a))/(1 + u^4 - 4*u^2*v + 4*v^2 + 4*u^2*v^2 + 16*u^4*v*a + 16*u^6*a^2)}

geo[moebiusstrip[a_]][p_,q_,u_,v_]:=
	{p,q,
	(-2*p*q*(3*v + 4*Cos[u/2] +
	2*v*Cos[u]))/(4 + 3*v^2 + 8*v*Cos[u/2] + 2*v^2*Cos[u]) +
	 (2*p^2*v*(1 + v*Cos[u/2])*Sin[u/2])/
	(4 + 3*v^2 + 8*v*Cos[u/2] + 2*v^2*Cos[u]),
	-(p^2*((-3*v)/4 - Cos[u/2] - (v*Cos[u])/2))}

geo[monkeysaddle][p_,q_,u_,v_]:=
	{p,q,
	(-18*p^2*u*(u^2 - v^2))/(1 + 9*u^4 + 18*u^2*v^2 + 9*v^4) -
	(18*q^2*u*(-u^2 + v^2))/(1 + 9*u^4 + 18*u^2*v^2 + 9*v^4) -
	(36*p*q*v*(-u^2 + v^2))/(1 + 9*u^4 + 18*u^2*v^2 + 9*v^4),
	(36*p^2*u^2*v)/(1 + 9*u^4 + 18*u^2*v^2 + 9*v^4) - (36*q^2*u^2*v)/
	(1 + 9*u^4 + 18*u^2*v^2 + 9*v^4) -
	(72*p*q*u*v^2)/(1 + 9*u^4 + 18*u^2*v^2 + 9*v^4)}

geo[paraboloid[a_,b_,c_]][p_,q_,u_,v_]:=
	{p,q,
	(-2*p^2*a*(2*u*a + v*c))/
	(1 + 4*u^2*a^2 + 4*v^2*b^2 + 4*u*v*a*c + 4*u*v*b*c +
	 u^2*c^2 + v^2*c^2) - (2*q^2*b*(2*u*a + v*c))/
	(1 + 4*u^2*a^2 + 4*v^2*b^2 + 4*u*v*a*c + 4*u*v*b*c +
	u^2*c^2 + v^2*c^2) -
	(2*p*q*c*(2*u*a + v*c))/
	(1 + 4*u^2*a^2 + 4*v^2*b^2 + 4*u*v*a*c +
	4*u*v*b*c + u^2*c^2 + v^2*c^2),
	(-2*p^2*a*(2*v*b + u*c))/
	(1 + 4*u^2*a^2 + 4*v^2*b^2 + 4*u*v*a*c + 4*u*v*b*c +
	 u^2*c^2 + v^2*c^2) -
	(2*q^2*b*(2*v*b + u*c))/
	(1 + 4*u^2*a^2 + 4*v^2*b^2 + 4*u*v*a*c + 4*u*v*b*c +
	u^2*c^2 + v^2*c^2) -
	(2*p*q*c*(2*v*b + u*c))/
	(1 + 4*u^2*a^2 + 4*v^2*b^2 + 4*u*v*a*c + 4*u*v*b*c +
	u^2*c^2 + v^2*c^2)}

geo[perturbedms[a_]][p_,q_,u_,v_]:=
	{p,q,
	(-2*q^2*(-u^3 + u*v^2 + u^2*a - 2*u^2*v*a - v^2*a + 2*u*v*a^2))/
	(1 + u^4 + 2*u^2*v^2 + v^4 - 8*u*v^2*a - 4*u*v^3*a + u^4*a^2 +
         4*u^2*v*a^2 + 4*v^2*a^2 + 4*u^2*v^2*a^2) -
	(4*p*q*(-(u^2*v) + v^3 + u^3*a - 3*u*v^2*a + 2*u^2*v*a^2))/
	(1 + u^4 + 2*u^2*v^2 + v^4 - 8*u*v^2*a - 4*u*v^3*a +
	u^4*a^2 + 4*u^2*v*a^2 + 4*v^2*a^2 + 4*u^2*v^2*a^2) -
	(2*p^2*(u^3 - u*v^2 + 3*u^2*v*a - v^3*a + 2*u*v^2*a^2))/
	(1 + u^4 + 2*u^2*v^2 + v^4 - 8*u*v^2*a - 4*u*v^3*a +
	u^4*a^2 + 4*u^2*v*a^2 + 4*v^2*a^2 + 4*u^2*v^2*a^2),
	(-2*q^2*(-u + a)*(-2*u*v + u^2*a + 2*v*a))/
	(1 + u^4 + 2*u^2*v^2 + v^4 - 8*u*v^2*a - 4*u*v^3*a +
	u^4*a^2 + 4*u^2*v*a^2 + 4*v^2*a^2 + 4*u^2*v^2*a^2) -
	(4*p*q*(2*u*v^2 - 3*u^2*v*a - 2*v^2*a + u^3*a^2 + 2*u*v*a^2))/
	(1 + u^4 + 2*u^2*v^2 + v^4 - 8*u*v^2*a - 4*u*v^3*a + u^4*a^2 +
 	4*u^2*v*a^2 + 4*v^2*a^2 + 4*u^2*v^2*a^2) -
	(2*p^2*(-2*u^2*v + u^3*a + 2*u*v*a - 2*u*v^2*a +
	u^2*v*a^2 + 2*v^2*a^2))/
	(1 + u^4 + 2*u^2*v^2 + v^4 - 8*u*v^2*a - 4*u*v^3*a +
	u^4*a^2 + 4*u^2*v*a^2 + 4*v^2*a^2 + 4*u^2*v^2*a^2)}

geo[pseudosphere[a_]][p_,q_,u_,v_]:={p,q,-2 p q Cot[v],
                           2 q^2 Csc[2 v] + p^2 Sin[v]^2 Tan[v]}

geo[pseudosphere[a_]][p_,q_,u_,v_]:=
	{p,q,-2*p*q*Cot[v],
	(q^2*Csc[v/2]*Sec[v/2]*Sec[v])/2 +
	4*p^2*Cos[v/2]^2*Sin[v/2]^2*Tan[v]}

geo[scherk[a_]][p_,q_,u_,v_]:=
	{p,q,
	-((p^2*a*Sec[u*a]^2*Tan[u*a])/(Sec[u*a]^2*Sec[v*a]^2 -
	 Tan[u*a]^2*Tan[v*a]^2)) +
	(q^2*a*Sec[v*a]^2*Tan[u*a])/(Sec[u*a]^2*Sec[v*a]^2 -
	Tan[u*a]^2*Tan[v*a]^2),
	(p^2*a*Sec[u*a]^2*Tan[v*a])/(Sec[u*a]^2*Sec[v*a]^2 -
	Tan[u*a]^2*Tan[v*a]^2) -
	(q^2*a*Sec[v*a]^2*Tan[v*a])/(Sec[u*a]^2*Sec[v*a]^2 -
	Tan[u*a]^2*Tan[v*a]^2)}

geo[shoe][p_,q_,u_,v_]:=
	{p,q,
	(q^2*u^2)/(1 + u^4 + v^2) - (2*p^2*u^3)/(1 + u^4 + v^2),
	 -((q^2*v)/(1 + u^4 + v^2)) + (2*p^2*u*v)/(1 + u^4 + v^2)}

geo[sphere[a_]][p_,q_,u_,v_]:={p,q,2*p*q*Tan[v],-(p^2*Sin[2*v])/2}

geo[torus[a_,b_,c_]][p_,q_,u_,v_]:=
	{p,q,
	(2*p*q*b*Sin[v])/(a + Cos[v]*b),
	(-2*p^2*b*(a + Cos[v]*b)*Sin[v])/(b^2 - Cos[2*v]*b^2 + c^2 +
	 Cos[2*v]*c^2) - (q^2*(b^2 - c^2)*Sin[2*v])/(b^2 -
	 Cos[2*v]*b^2 + c^2 + Cos[2*v]*c^2)}

geo[twisphere[a_,b_]][p_,q_,u_,v_]:=
	{p,q,
	(2*q^2*a*b*Sin[v])/(a^2 + Cos[2*v]*a^2 + b^2 - Cos[2*v]*b^2) +
	(2*p*q*a^2*Sin[2*v])/(a^2 + Cos[2*v]*a^2 + b^2 - Cos[2*v]*b^2) +
	(p^2*Cos[v]*a*b*Sin[2*v])/(a^2 + Cos[2*v]*a^2 + b^2 - Cos[2*v]*b^2),
	(-2*p*q*Cos[v]*a*b*Sin[2*v])/
	(a^2 + Cos[2*v]*a^2 + b^2 - Cos[2*v]*b^2) -
	(q^2*b^2*Sin[2*v])/(a^2 + Cos[2*v]*a^2 + b^2 - Cos[2*v]*b^2) -
	(p^2*(a^2/2 + (Cos[2*v]*a^2)/2 + b^2)*Sin[2*v])/
	(a^2 + Cos[2*v]*a^2 + b^2 - Cos[2*v]*b^2)}

geo[whitneyumbrella][p_,q_,u_,v_]:=
	{p,q,
	(4*q^2*u*v^2)/(u^2 + 4*v^2 + 4*v^4) -
	(8*p*q*v^3)/(u^2 + 4*v^2 + 4*v^4),
	(-2*p*q*u)/(u^2 + 4*v^2 + 4*v^4) -
	(4*q^2*v*(1 + v^2))/(u^2 + 4*v^2 + 4*v^4)}

End[]

EndPackage[]
