(*:Date: Tue Feb  3 04:40:04 EST 1998
*)

(*:Title: INVERCUR2D.m *)

(*:Author: Alfred Gray *)

(* Copyright 1996-1998 by Alfred Gray *)

(*:Package Version: 1.0 *)

(*:Mathematica Version: 2.2, 3.0 *)

(*:Summary: 
This package consists of formulas for the inversion curves of many plane curves.*)

BeginPackage["INVCUR2D`","CURVES`","CSPROGS`"]

inversioncurve::usage="t->inversioncurve[q,rho][alpha][t] is the inversion of a curve alpha with respect to a sphere in R^n of radius rho centered at q. t->inversioncurve[rho][alpha][t] is the inversion of a curve alpha with respect to a sphere in R^n of radius rho centered at the origin."

Begin["`Private`"]
inversioncurve[{p1_,p2_},rho_][agnesi[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + 2*a*Tan[t]))/
       ((-p2 + 2*a*Cos[t]^2)^2 + (-p1 + 2*a*Tan[t])^2), 
     p2 + (rho^2*(-p2 + 2*a*Cos[t]^2))/
       ((-p2 + 2*a*Cos[t]^2)^2 + (-p1 + 2*a*Tan[t])^2)}

inversioncurve[{p1_,p2_},rho_][archimedesspiral[n_,a_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*t^n^(-1)*Cos[t]))/
       ((-p1 + a*t^n^(-1)*Cos[t])^2 + 
         (-p2 + a*t^n^(-1)*Sin[t])^2), 
     p2 + (rho^2*(-p2 + a*t^n^(-1)*Sin[t]))/
       ((-p1 + a*t^n^(-1)*Cos[t])^2 + 
         (-p2 + a*t^n^(-1)*Sin[t])^2)}

inversioncurve[{p1_,p2_},rho_][astroid[n_,a_,b_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*Cos[t]^n))/
       ((-p1 + a*Cos[t]^n)^2 + (-p2 + b*Sin[t]^n)^2), 
     p2 + (rho^2*(-p2 + b*Sin[t]^n))/
       ((-p1 + a*Cos[t]^n)^2 + (-p2 + b*Sin[t]^n)^2)}

inversioncurve[{p1_,p2_},rho_][cardioid[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + 2*a*Cos[t]*(1 + Cos[t])))/
       ((-p1 + 2*a*Cos[t]*(1 + Cos[t]))^2 + 
         (-p2 + 2*a*(1 + Cos[t])*Sin[t])^2), 
     p2 + (rho^2*(-p2 + 2*a*(1 + Cos[t])*Sin[t]))/
       ((-p1 + 2*a*Cos[t]*(1 + Cos[t]))^2 + 
         (-p2 + 2*a*(1 + Cos[t])*Sin[t])^2)}

inversioncurve[{p1_,p2_},rho_][catenary[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*Cosh[t/a]))/
       ((-p2 + t)^2 + (-p1 + a*Cosh[t/a])^2), 
     p2 + (rho^2*(-p2 + t))/
       ((-p2 + t)^2 + (-p1 + a*Cosh[t/a])^2)}

inversioncurve[{p1_,p2_},rho_][cayleysextic[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + 4*a*Cos[t/2]^4*(-1 + 2*Cos[t])))/
       ((-p1 + 4*a*Cos[t/2]^4*(-1 + 2*Cos[t]))^2 + 
         (-p2 + 4*a*Cos[t/2]^3*Sin[(3*t)/2])^2), 
     p2 + (rho^2*(-p2 + 4*a*Cos[t/2]^3*Sin[(3*t)/2]))/
       ((-p1 + 4*a*Cos[t/2]^4*(-1 + 2*Cos[t]))^2 + 
         (-p2 + 4*a*Cos[t/2]^3*Sin[(3*t)/2])^2)}

inversioncurve[{p1_,p2_},rho_][circle[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*Cos[t]))/
       ((-p1 + a*Cos[t])^2 + (-p2 + a*Sin[t])^2), 
     p2 + (rho^2*(-p2 + a*Sin[t]))/
       ((-p1 + a*Cos[t])^2 + (-p2 + a*Sin[t])^2)}

inversioncurve[{p1_,p2_},rho_][circleinvolute[1,a_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*(Cos[t] + t*Sin[t])))/
       ((-p2 + a*(-(t*Cos[t]) + Sin[t]))^2 + 
         (-p1 + a*(Cos[t] + t*Sin[t]))^2), 
     p2 + (rho^2*(-p2 + a*(-(t*Cos[t]) + Sin[t])))/
       ((-p2 + a*(-(t*Cos[t]) + Sin[t]))^2 + 
         (-p1 + a*(Cos[t] + t*Sin[t]))^2)}

inversioncurve[{p1_,p2_},rho_][clothoid[1,a_]][t_]:= 
    {p1 + (rho^2*(-p1 + 
           a*Pi^(1/2)*FresnelS[t/Pi^(1/2)]))/
       ((-p2 + a*Pi^(1/2)*FresnelC[t/Pi^(1/2)])^2 + 
         (-p1 + a*Pi^(1/2)*FresnelS[t/Pi^(1/2)])^2), 
     p2 + (rho^2*(-p2 + 
           a*Pi^(1/2)*FresnelC[t/Pi^(1/2)]))/
       ((-p2 + a*Pi^(1/2)*FresnelC[t/Pi^(1/2)])^2 + 
         (-p1 + a*Pi^(1/2)*FresnelS[t/Pi^(1/2)])^2)}

inversioncurve[{p1_,p2_},rho_][clothoidprime[n_,a_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*Sin[t^(1 + n)/(1 + n)]))/
       ((-p2 + a*Cos[t^(1 + n)/(1 + n)])^2 + 
         (-p1 + a*Sin[t^(1 + n)/(1 + n)])^2), 
     p2 + (rho^2*(-p2 + a*Cos[t^(1 + n)/(1 + n)]))/
       ((-p2 + a*Cos[t^(1 + n)/(1 + n)])^2 + 
         (-p1 + a*Sin[t^(1 + n)/(1 + n)])^2)}

inversioncurve[{p1_,p2_},rho_][cnccoprofile[pm, a, b]][t]:= 
    {p1 + (rho^2*(-p1 + b*pm*Sinh[t/a]))/
       ((-p2 - I*(a^2 - b^2)^(1/2)*
             EllipticE[(I*t)/a, -(b^2/(a^2 - b^2)^(1/2))]
            )^2 + (-p1 + b*pm*Sinh[t/a])^2), 
     p2 + (rho^2*(-p2 - 
           I*(a^2 - b^2)^(1/2)*
            EllipticE[(I*t)/a, -(b^2/(a^2 - b^2)^(1/2))])
         )/((-p2 - I*(a^2 - b^2)^(1/2)*
             EllipticE[(I*t)/a, -(b^2/(a^2 - b^2)^(1/2))]
            )^2 + (-p1 + b*pm*Sinh[t/a])^2)}

inversioncurve[{p1_,p2_},rho_][cnchyprofile[pm, a, b]][t]:= 
    {p1 + (rho^2*(-p1 + b*pm*Cosh[t/a]))/
       ((-p1 + b*pm*Cosh[t/a])^2 + 
         (-p2 - I*a*EllipticE[(I*t)/a, -(b^2/a^2)])^2), 
     p2 + (rho^2*(-p2 - 
           I*a*EllipticE[(I*t)/a, -(b^2/a^2)]))/
       ((-p1 + b*pm*Cosh[t/a])^2 + 
         (-p2 - I*a*EllipticE[(I*t)/a, -(b^2/a^2)])^2)}

inversioncurve[{p1_,p2_},rho_][cochleoid[n_,a_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*Cos[t]*(Sin[t]/t)^n))/
       ((-p1 + a*Cos[t]*(Sin[t]/t)^n)^2 + 
         (-p2 + a*Sin[t]*(Sin[t]/t)^n)^2), 
     p2 + (rho^2*(-p2 + a*Sin[t]*(Sin[t]/t)^n))/
       ((-p1 + a*Cos[t]*(Sin[t]/t)^n)^2 + 
         (-p2 + a*Sin[t]*(Sin[t]/t)^n)^2)}

inversioncurve[{p1_,p2_},rho_][conchoid[a_,b_]][t_]:= 
    {p1 + (rho^2*(b - p1 + a*Cos[t]))/
       ((b - p1 + a*Cos[t])^2 + 
         (-p2 + a*Sin[t] + b*Tan[t])^2), 
     p2 + (rho^2*(-p2 + a*Sin[t] + b*Tan[t]))/
       ((b - p1 + a*Cos[t])^2 + 
         (-p2 + a*Sin[t] + b*Tan[t])^2)}

inversioncurve[{p1_,p2_},rho_][cothspiral[a_]][t_]:= 
    {p1 + (rho^2*(-p1 - 
           Sinh[2*t]/(Cos[2*a*t] - Cosh[2*t])))/
       ((-p2 + Sin[2*a*t]/(Cos[2*a*t] - Cosh[2*t]))^2 + 
         (-p1 - Sinh[2*t]/(Cos[2*a*t] - Cosh[2*t]))^2), 
     p2 + (rho^2*(-p2 + 
           Sin[2*a*t]/(Cos[2*a*t] - Cosh[2*t])))/
       ((-p2 + Sin[2*a*t]/(Cos[2*a*t] - Cosh[2*t]))^2 + 
         (-p1 - Sinh[2*t]/(Cos[2*a*t] - Cosh[2*t]))^2)}

inversioncurve[{p1_,p2_},rho_][cpcprofile[pm, a, b]][t]:= 
    {p1 + (rho^2*(-p1 + b*pm*Cos[t/a]))/
       ((-p1 + b*pm*Cos[t/a])^2 + 
         (-p2 + a*EllipticE[t/a, b^2/a^2])^2), 
     p2 + (rho^2*(-p2 + a*EllipticE[t/a, b^2/a^2]))/
       ((-p1 + b*pm*Cos[t/a])^2 + 
         (-p2 + a*EllipticE[t/a, b^2/a^2])^2)}

inversioncurve[{p1_,p2_},rho_][cubicparabola[a_,b_,c_,d_]][t_]:= 
    {p1 + (rho^2*(-p1 + t))/
       ((-p1 + t)^2 + (d - p2 + c*t + b*t^2 + a*t^3)^2), 
     p2 + (rho^2*(d - p2 + c*t + b*t^2 + a*t^3))/
       ((-p1 + t)^2 + (d - p2 + c*t + b*t^2 + a*t^3)^2)}

    inversioncurve[{p1_,p2_},rho_][cycloid[a_,b_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*t - b*Sin[t]))/
       ((a - p2 - b*Cos[t])^2 + (-p1 + a*t - b*Sin[t])^2)
      , p2 + (rho^2*(a - p2 - b*Cos[t]))/
       ((a - p2 - b*Cos[t])^2 + (-p1 + a*t - b*Sin[t])^2)}

inversioncurve[{p1_,p2_},rho_][deltoid[a_]][t_]:= 
    {p1 + (rho^2*(-a - p1 + 2*a*Cos[t]*(1 + Cos[t])))/
       ((-a - p1 + 2*a*Cos[t]*(1 + Cos[t]))^2 + 
         (-p2 + 2*a*(1 - Cos[t])*Sin[t])^2), 
     p2 + (rho^2*(-p2 + 2*a*(1 - Cos[t])*Sin[t]))/
       ((-a - p1 + 2*a*Cos[t]*(1 + Cos[t]))^2 + 
         (-p2 + 2*a*(1 - Cos[t])*Sin[t])^2)}

inversioncurve[{p1_,p2_},rho_][diamond[n_,a_,b_]][t_]:= 
    {p1 + (rho^2*(-p1 + 
           a*Cos[t]*(Cos[t]^2)^((-1 + n)/2)))/
       ((-p1 + a*Cos[t]*(Cos[t]^2)^((-1 + n)/2))^2 + 
         (-p2 + b*Sin[t]*(Sin[t]^2)^((-1 + n)/2))^2), 
     p2 + (rho^2*(-p2 + 
           b*Sin[t]*(Sin[t]^2)^((-1 + n)/2)))/
       ((-p1 + a*Cos[t]*(Cos[t]^2)^((-1 + n)/2))^2 + 
         (-p2 + b*Sin[t]*(Sin[t]^2)^((-1 + n)/2))^2)}

inversioncurve[{p1_,p2_},rho_][eight][t]:= 
    {p1 + (rho^2*(-p1 + Sin[t]))/
       ((-p1 + Sin[t])^2 + (-p2 + Cos[t]*Sin[t])^2), 
     p2 + (rho^2*(-p2 + Cos[t]*Sin[t]))/
       ((-p1 + Sin[t])^2 + (-p2 + Cos[t]*Sin[t])^2)}

inversioncurve[{p1_,p2_},rho_][ellipse[a_,b_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*Cos[t]))/
       ((-p1 + a*Cos[t])^2 + (-p2 + b*Sin[t])^2), 
     p2 + (rho^2*(-p2 + b*Sin[t]))/
       ((-p1 + a*Cos[t])^2 + (-p2 + b*Sin[t])^2)}

inversioncurve[{p1_,p2_},rho_][epicycloid[a_,b_]][t_]:= 
    {p1 + (rho^2*(-p1 + (a + b)*Cos[t] - 
           b*Cos[((a + b)*t)/b]))/
       ((-p1 + (a + b)*Cos[t] - b*Cos[((a + b)*t)/b])^
          2 + (-p2 + (a + b)*Sin[t] - 
            b*Sin[((a + b)*t)/b])^2), 
     p2 + (rho^2*(-p2 + (a + b)*Sin[t] - 
           b*Sin[((a + b)*t)/b]))/
       ((-p1 + (a + b)*Cos[t] - b*Cos[((a + b)*t)/b])^
          2 + (-p2 + (a + b)*Sin[t] - 
            b*Sin[((a + b)*t)/b])^2)}

inversioncurve[{p1_,p2_},rho_][epitrochoid[a_,b_,h_]][t_]:= 
    {p1 + (rho^2*(-p1 + (a + b)*Cos[t] - 
           h*Cos[((a + b)*t)/b]))/
       ((-p1 + (a + b)*Cos[t] - h*Cos[((a + b)*t)/b])^
          2 + (-p2 + (a + b)*Sin[t] - 
            h*Sin[((a + b)*t)/b])^2), 
     p2 + (rho^2*(-p2 + (a + b)*Sin[t] - 
           h*Sin[((a + b)*t)/b]))/
       ((-p1 + (a + b)*Cos[t] - h*Cos[((a + b)*t)/b])^
          2 + (-p2 + (a + b)*Sin[t] - 
            h*Sin[((a + b)*t)/b])^2)}

inversioncurve[{p1_,p2_},rho_][cissoid[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + (2*a*t^2)/(1 + t^2)))/
       ((-p1 + (2*a*t^2)/(1 + t^2))^2 + 
         (-p2 + (2*a*t^3)/(1 + t^2))^2), 
     p2 + (rho^2*(-p2 + (2*a*t^3)/(1 + t^2)))/
       ((-p1 + (2*a*t^2)/(1 + t^2))^2 + 
         (-p2 + (2*a*t^3)/(1 + t^2))^2)}

inversioncurve[{p1_,p2_},rho_][fermatspiral[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*t^(1/2)*Cos[t]))/
       ((-p1 + a*t^(1/2)*Cos[t])^2 + 
         (-p2 + a*t^(1/2)*Sin[t])^2), 
     p2 + (rho^2*(-p2 + a*t^(1/2)*Sin[t]))/
       ((-p1 + a*t^(1/2)*Cos[t])^2 + 
         (-p2 + a*t^(1/2)*Sin[t])^2)}

inversioncurve[{p1_,p2_},rho_][folium][t]:= 
    {p1 + (rho^2*(-p1 + (3*t)/(1 + t^3)))/
       ((-p1 + (3*t)/(1 + t^3))^2 + 
         (-p2 + (3*t^2)/(1 + t^3))^2), 
     p2 + (rho^2*(-p2 + (3*t^2)/(1 + t^3)))/
       ((-p1 + (3*t)/(1 + t^3))^2 + 
         (-p2 + (3*t^2)/(1 + t^3))^2)}

inversioncurve[{p1_,p2_},rho_][hyperbola[a_,b_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*Cosh[t]))/
       ((-p1 + a*Cosh[t])^2 + (-p2 + b*Sinh[t])^2), 
     p2 + (rho^2*(-p2 + b*Sinh[t]))/
       ((-p1 + a*Cosh[t])^2 + (-p2 + b*Sinh[t])^2)}

inversioncurve[{p1_,p2_},rho_][hyperbolicspiral[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + (a*Cos[t])/t))/
       ((-p1 + (a*Cos[t])/t)^2 + (-p2 + (a*Sin[t])/t)^2),
       p2 + (rho^2*(-p2 + (a*Sin[t])/t))/
       ((-p1 + (a*Cos[t])/t)^2 + (-p2 + (a*Sin[t])/t)^2)}

inversioncurve[{p1_,p2_},rho_][
      hypocycloid[a_,b_]][t_]:= 
    {p1 + (rho^2*(-p1 + (a - b)*Cos[t] + 
           b*Cos[((a - b)*t)/b]))/
       ((-p1 + (a - b)*Cos[t] + b*Cos[((a - b)*t)/b])^
          2 + (-p2 + (a - b)*Sin[t] - 
            b*Sin[((a - b)*t)/b])^2), 
     p2 + (rho^2*(-p2 + (a - b)*Sin[t] - 
           b*Sin[((a - b)*t)/b]))/
       ((-p1 + (a - b)*Cos[t] + b*Cos[((a - b)*t)/b])^
          2 + (-p2 + (a - b)*Sin[t] - 
            b*Sin[((a - b)*t)/b])^2)}

inversioncurve[{p1_,p2_},rho_][hypotrochoid[a_,b_,h_]][t_]:= 
    {p1 + (rho^2*(-p1 + (a - b)*Cos[t] + 
           h*Cos[((a - b)*t)/b]))/
       ((-p1 + (a - b)*Cos[t] + h*Cos[((a - b)*t)/b])^
          2 + (-p2 + (a - b)*Sin[t] - 
            h*Sin[((a - b)*t)/b])^2), 
     p2 + (rho^2*(-p2 + (a - b)*Sin[t] - 
           h*Sin[((a - b)*t)/b]))/
       ((-p1 + (a - b)*Cos[t] + h*Cos[((a - b)*t)/b])^
          2 + (-p2 + (a - b)*Sin[t] - 
            h*Sin[((a - b)*t)/b])^2)}

inversioncurve[{p1_,p2_},rho_][lemniscate[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + (a*Cos[t])/(1 + Sin[t]^2)))/
       ((-p1 + (a*Cos[t])/(1 + Sin[t]^2))^2 + 
         (-p2 + (a*Cos[t]*Sin[t])/(1 + Sin[t]^2))^2), 
     p2 + (rho^2*(-p2 + 
           (a*Cos[t]*Sin[t])/(1 + Sin[t]^2)))/
       ((-p1 + (a*Cos[t])/(1 + Sin[t]^2))^2 + 
         (-p2 + (a*Cos[t]*Sin[t])/(1 + Sin[t]^2))^2)}

inversioncurve[{p1_,p2_},rho_][limacon[a_,b_]][t_]:= 
    {p1 + (rho^2*(-p1 + Cos[t]*(b + 2*a*Cos[t])))/
       ((-p1 + Cos[t]*(b + 2*a*Cos[t]))^2 + 
         (-p2 + (b + 2*a*Cos[t])*Sin[t])^2), 
     p2 + (rho^2*(-p2 + (b + 2*a*Cos[t])*Sin[t]))/
       ((-p1 + Cos[t]*(b + 2*a*Cos[t]))^2 + 
         (-p2 + (b + 2*a*Cos[t])*Sin[t])^2)}

inversioncurve[{p1_,p2_},rho_][line[a1_,b1_,a2_,b2_]][t_]:= 
    {p1 + (rho^2*(a2 - p1 + a1*t))/
       ((a2 - p1 + a1*t)^2 + (b2 - p2 + b1*t)^2), 
     p2 + (rho^2*(b2 - p2 + b1*t))/
       ((a2 - p1 + a1*t)^2 + (b2 - p2 + b1*t)^2)}

inversioncurve[{p1_,p2_},rho_][lissajous[n_,d_,a_,b_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*Sin[d + n*t]))/
       ((-p2 + b*Sin[t])^2 + (-p1 + a*Sin[d + n*t])^2), 
     p2 + (rho^2*(-p2 + b*Sin[t]))/
       ((-p2 + b*Sin[t])^2 + (-p1 + a*Sin[d + n*t])^2)}

inversioncurve[{p1_,p2_},rho_][lituus[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + 
           (a*t*Cos[(t^2)^(1/2)])/(t^2)^(3/4)))/
       ((-p1 + (a*t*Cos[(t^2)^(1/2)])/(t^2)^(3/4))^2 + 
         (-p2 + (a*t*Sin[(t^2)^(1/2)])/(t^2)^(3/4))^2), 
     p2 + (rho^2*(-p2 + 
           (a*t*Sin[(t^2)^(1/2)])/(t^2)^(3/4)))/
       ((-p1 + (a*t*Cos[(t^2)^(1/2)])/(t^2)^(3/4))^2 + 
         (-p2 + (a*t*Sin[(t^2)^(1/2)])/(t^2)^(3/4))^2)}

inversioncurve[{p1_,p2_},rho_][logspiral[a_,b_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*E^(b*t)*Cos[t]))/
       ((-p1 + a*E^(b*t)*Cos[t])^2 + 
         (-p2 + a*E^(b*t)*Sin[t])^2), 
     p2 + (rho^2*(-p2 + a*E^(b*t)*Sin[t]))/
       ((-p1 + a*E^(b*t)*Cos[t])^2 + 
         (-p2 + a*E^(b*t)*Sin[t])^2)}

inversioncurve[{p1_,p2_},rho_][nephroid[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*(3*Cos[t] - Cos[3*t])))/
       ((-p1 + a*(3*Cos[t] - Cos[3*t]))^2 + 
         (-p2 + a*(3*Sin[t] - Sin[3*t]))^2), 
     p2 + (rho^2*(-p2 + a*(3*Sin[t] - Sin[3*t])))/
       ((-p1 + a*(3*Cos[t] - Cos[3*t]))^2 + 
         (-p2 + a*(3*Sin[t] - Sin[3*t]))^2)}

inversioncurve[{p1_,p2_},rho_][nielsenspiral[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*CosIntegral[t]))/
       ((-p1 + a*CosIntegral[t])^2 + 
         (-p2 + a*SinIntegral[t])^2), 
     p2 + (rho^2*(-p2 + a*SinIntegral[t]))/
       ((-p1 + a*CosIntegral[t])^2 + 
         (-p2 + a*SinIntegral[t])^2)}

inversioncurve[{p1_,p2_},rho_][parabola[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + 2*a*t))/
       ((-p1 + 2*a*t)^2 + (-p2 + a*t^2)^2), 
     p2 + (rho^2*(-p2 + a*t^2))/
       ((-p1 + 2*a*t)^2 + (-p2 + a*t^2)^2)}

inversioncurve[{p1_,p2_},rho_][piriform[a_,b_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*(1 + Sin[t])))/
       ((-p1 + a*(1 + Sin[t]))^2 + 
         (-p2 + b*Cos[t]*(1 + Sin[t]))^2), 
     p2 + (rho^2*(-p2 + b*Cos[t]*(1 + Sin[t])))/
       ((-p1 + a*(1 + Sin[t]))^2 + 
         (-p2 + b*Cos[t]*(1 + Sin[t]))^2)}

inversioncurve[{p1_,p2_},rho_][rose[n_,a_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*Cos[t]*Cos[n*t]))/
       ((-p1 + a*Cos[t]*Cos[n*t])^2 + 
         (-p2 + a*Cos[n*t]*Sin[t])^2), 
     p2 + (rho^2*(-p2 + a*Cos[n*t]*Sin[t]))/
       ((-p1 + a*Cos[t]*Cos[n*t])^2 + 
         (-p2 + a*Cos[n*t]*Sin[t])^2)}

inversioncurve[{p1_,p2_},rho_][sc][t]:= 
    {p1 + (rho^2*(-p1 + t^2))/
       ((-p1 + t^2)^2 + (-p2 + t^3)^2), 
     p2 + (rho^2*(-p2 + t^3))/
       ((-p1 + t^2)^2 + (-p2 + t^3)^2)}

inversioncurve[{p1_,p2_},rho_][strophoid[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + (a*(-1 + t^2))/(1 + t^2)))/
       ((-p1 + (a*(-1 + t^2))/(1 + t^2))^2 + 
         (-p2 + (a*t*(-1 + t^2))/(1 + t^2))^2), 
     p2 + (rho^2*(-p2 + (a*t*(-1 + t^2))/(1 + t^2)))/
       ((-p1 + (a*(-1 + t^2))/(1 + t^2))^2 + 
         (-p2 + (a*t*(-1 + t^2))/(1 + t^2))^2)}

inversioncurve[{p1_,p2_},rho_][talbot[a_,b_,f_]][t_]:= 
    {p1 + (rho^2*(-p1 + Cos[t]*(a + (f^2*Sin[t]^2)/a)))/
     ((p2 - (Sin[t]*(a^2 - 2*f^2 + f^2*Sin[t]^2))/b)^2 + 
     (p1 - Cos[t]*(a + (f^2*Sin[t]^2)/a))^2), 
      p2 + (rho^2*(-p2 + (Sin[t]*(a^2 - 2*f^2 + f^2*Sin[t]^2))/b))/
     ((p2 - (Sin[t]*(a^2 - 2*f^2 + f^2*Sin[t]^2))/b)^2 + 
     (p1 - Cos[t]*(a + (f^2*Sin[t]^2)/a))^2)}

inversioncurve[{p1_,p2_},rho_][talbot[a,b,f]][t_]:= 
    {p1 + (rho^2*(-p1 + Cos[t]*(a + (f^2*Sin[t]^2)/a)))/
    ((p2 - (Sin[t]*(a^2 - 2*f^2 + f^2*Sin[t]^2))/b)^2 + 
      (p1 - Cos[t]*(a + (f^2*Sin[t]^2)/a))^2), 
      p2 + (rho^2*(-p2 + (Sin[t]*(a^2 - 2*f^2 + f^2*Sin[t]^2))/b))/
    ((p2 - (Sin[t]*(a^2 - 2*f^2 + f^2*Sin[t]^2))/b)^2 + 
      (p1 - Cos[t]*(a + (f^2*Sin[t]^2)/a))^2)}

inversioncurve[{p1_,p2_},rho_][tanhspiral[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + 
           Sinh[2*t]/(Cos[2*a*t] + Cosh[2*t])))/
       ((-p2 + Sin[2*a*t]/(Cos[2*a*t] + Cosh[2*t]))^2 + 
         (-p1 + Sinh[2*t]/(Cos[2*a*t] + Cosh[2*t]))^2), 
     p2 + (rho^2*(-p2 + 
           Sin[2*a*t]/(Cos[2*a*t] + Cosh[2*t])))/
       ((-p2 + Sin[2*a*t]/(Cos[2*a*t] + Cosh[2*t]))^2 + 
         (-p1 + Sinh[2*t]/(Cos[2*a*t] + Cosh[2*t]))^2)}

inversioncurve[{p1_,p2_},rho_][tractrix[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*Sin[t]))/
       ((-p2 + a*(Cos[t] + Log[Tan[t/2]]))^2 + 
         (-p1 + a*Sin[t])^2), 
     p2 + (rho^2*(-p2 + a*(Cos[t] + Log[Tan[t/2]])))/
       ((-p2 + a*(Cos[t] + Log[Tan[t/2]]))^2 + 
         (-p1 + a*Sin[t])^2)}
(*
inversioncurve[{p1_,p2_},rho_][triangle][t]:= 
    {p1 + (rho^2*(-p1 + 
           If[t < 1, t*{1, 0}, 
            If[t < 2, {1, 0} + 
              (t - 1)*{-2^(-1), Sqrt[3]/2}, 
             {1/2, Sqrt[3]/2} + 
              (t - 2)*{-2^(-1), -(Sqrt[3]/2)}]]))/
       ((-p1 + If[t < 1, t*{1, 0}, 
             If[t < 2, 
              {1, 0} + (t - 1)*{-2^(-1), Sqrt[3]/2}, 
              {1/2, Sqrt[3]/2} + 
               (t - 2)*{-2^(-1), -(Sqrt[3]/2)}]])^2 + 
         (-p2 + If[t < 1, t*{1, 0}, 
             If[t < 2, 
              {1, 0} + (t - 1)*{-2^(-1), Sqrt[3]/2}, 
              {1/2, Sqrt[3]/2} + 
               (t - 2)*{-2^(-1), -(Sqrt[3]/2)}]])^2), 
     p2 + (rho^2*(-p2 + 
           If[t < 1, t*{1, 0}, 
            If[t < 2, {1, 0} + 
              (t - 1)*{-2^(-1), Sqrt[3]/2}, 
             {1/2, Sqrt[3]/2} + 
              (t - 2)*{-2^(-1), -(Sqrt[3]/2)}]]))/
       ((-p1 + If[t < 1, t*{1, 0}, 
             If[t < 2, 
              {1, 0} + (t - 1)*{-2^(-1), Sqrt[3]/2}, 
              {1/2, Sqrt[3]/2} + 
               (t - 2)*{-2^(-1), -(Sqrt[3]/2)}]])^2 + 
         (-p2 + If[t < 1, t*{1, 0}, 
             If[t < 2, 
              {1, 0} + (t - 1)*{-2^(-1), Sqrt[3]/2}, 
              {1/2, Sqrt[3]/2} + 
               (t - 2)*{-2^(-1), -(Sqrt[3]/2)}]])^2)}
*)

inversioncurve[{p1_,p2_},rho_][trisectrix[a_]][t_]:= 
    {p1 + (rho^2*(-p1 + a*(1 - 4*Cos[t]^2)))/
       ((-p1 + a*(1 - 4*Cos[t]^2))^2 + 
         (-p2 + a*(1 - 4*Cos[t]^2)*Tan[t])^2), 
     p2 + (rho^2*(-p2 + a*(1 - 4*Cos[t]^2)*Tan[t]))/
       ((-p1 + a*(1 - 4*Cos[t]^2))^2 + 
         (-p2 + a*(1 - 4*Cos[t]^2)*Tan[t])^2)}

inversioncurve[{p1_,p2_},rho_][zcprofile[pm_,a_,c_]][t_]:= 
    {p1 + (rho^2*(-p1 + t))/
       ((-p1 + t)^2 + (-p2 + pm*(-c^2 + a^2*t^2)^(1/2) + 
            c*ArcSin[c/(a*t)])^2), 
     p2 + (rho^2*(-p2 + pm*(-c^2 + a^2*t^2)^(1/2) + 
           c*ArcSin[c/(a*t)]))/
       ((-p1 + t)^2 + (-p2 + pm*(-c^2 + a^2*t^2)^(1/2) + 
            c*ArcSin[c/(a*t)])^2)}

End[]

EndPackage[]
