(*:Date: Sun Jun 14 10:21:24 BST 1998
*)

(*:Title: KAPPA2.m *)

(*:Author: Alfred Gray *)

(* Copyright 1996-1998 by Alfred Gray *)

(*:Package Version: 1.0 *)

(*:Mathematica Version: 2.2, 3.0 *)

(*:Summary: 
This package consists of formulas for the signed curvature, equiaffine signed curvature and arclength functions of many plane curves.*)

BeginPackage["KAPPA2`","CURVES`","CSPROGS`"]

    arclength::usage="t->arclength[alpha][t] is the arc length function of the curve alpha:(a,b)->R^n computed using Integrate."

    kappa2::usage="t->kappa2[alpha][t] is the signed curvature of the plane curve alpha."

    kappa2ea::usage="t->kappa2ea[alpha][t] is the signed equiaffine curvature of the plane curve alpha."

Begin["`Private`"]

kappa2[agnesi[a_]][t_]:=
(2*Sqrt[2]*(1 - 2*Cos[2*t])*Sec[t]^2)/
  (a*(1 - Cos[4*t] + 2*Sec[t]^4)^(3/2))

kappa2ea[agnesi[a_]][t_]:=
((-4 + 3*Cos[2*t] - 2*Cos[4*t] + Cos[6*t])*Sec[t]^(2/3))/
  (54*3^(1/3)*a^(4/3)*(-1 + 2*Cos[2*t])^(8/3))

kappa2[airyspiral[a_]][t_]:=
t*(AiryAiPrime[t]*AiryBi[t] - AiryAi[t]*AiryBiPrime[t])/
   (a*(AiryAiPrime[t]^2 + AiryBiPrime[t]^2)^(3/2))

kappa2ea[airyspiral[a_]][t_]:=
-(5 + 9*t^3)/
  (243*3^(1/3)*t^2*(a^2*t*AiryAiPrime[t]*AiryBi[t] - 
       a^2*t*AiryAi[t]*AiryBiPrime[t])^(2/3))

arclength[alysoid[a_,b_,c_]][s_]:= s

kappa2[alysoid[a_,b_,c_]][s_]:= -c/(a*b + s^2)

kappa2[archimedesspiral[n_,a_]][t_]:=
 (n*t^(1 - n^(-1))*(1 + n + n^2*t^2))/
  (a*(1 + n^2*t^2)^(3/2))

kappa2ea[archimedesspiral[n_,a_]][t_]:=
(t^(-2 - (2*(-2 + 2/n))/3)*
    (1 + 3*n + 9*n^6*t^6 + n^2*(1 + 11*t^2) + 
      3*n^5*t^2*(6 + 13*t^2) + n^3*(-3 + 42*t^2) + 
      n^4*(-2 + 49*t^2 + 19*t^4)))/
  (243*3^(1/3)*a^(4/3)*n^(2/3)*(1 + n + n^2*t^2)^(8/3))
    
kappa2[astroid[n_,a_,b_]][t_]:=
(a*b*(2 - n)*n^2*Cos[t]^(n - 1)*Sin[t]^(n - 1))/
  (a^2*n^2*Cos[t]^(-2 + 2*n)*Sin[t]^2 + 
     b^2*n^2*Cos[t]^2*Sin[t]^(-2 + 2*n))^(3/2)

kappa2ea[astroid[n_,a_,b_]][t_]:=
((-4 + n)*(14 - 17*n + (2 + n)*Cos[4*t]))/
  (1944*3^(1/3)*a^(2/3)*b^(2/3)*(-2 + n)^(2/3)*n^(4/3)*
    Cos[t]^((2*(2 + n))/3)*Sin[t]^((2*(2 + n))/3))

kappa2[bellcurve[mean_,variance_]][t_]:=
(mean^2 - 2*mean*t + t^2 - variance^2)/
   (E^((-mean + t)^2/(2*variance^2))*(2*Pi)^(1/2)*
     (1 + (-mean + t)^2/
 (2*E^((-mean + t)^2/variance^2)*
 Pi*variance^6))^(3/2)*variance^5)

kappa2ea[bellcurve[mean_,variance_]][t_]:=
-(((2*Pi)/3)^(1/3)*(2*mean^6 - 12*mean^5*t + 2*t^6 - 
       9*t^4*variance^2 + 18*t^2*variance^4 + 
       9*variance^6 + 3*mean^4*(10*t^2 - 3*variance^2) + 
       mean^3*(-40*t^3 + 36*t*variance^2) + 
       6*mean^2*(5*t^4 - 9*t^2*variance^2 + 3*variance^4) - 
       12*mean*(t^5 - 3*t^3*variance^2 + 3*t*variance^4)))/
    (243*variance^4*(mean^2 - 2*mean*t + t^2 - 
       variance^2)^2*((mean^2 - 2*mean*t + t^2 - 
         variance^2)/
       (E^((mean - t)^2/(2*variance^2))*variance^5))^(2/3))

kappa2[bicorn[a_]][t_]:=
(6*Sqrt[2]*(-2 + Cos[t])^3*(-2 + 3*Cos[t])*Sec[t])/
   (a*(73 - 80*Cos[t] + 9*Cos[2*t])^(3/2))

kappa2ea[bicorn[a_]][t_]:=
(7658 - 12992*Cos[t] + 6335*Cos[2*t] - 2080*Cos[3*t] + 
    806*Cos[4*t] - 288*Cos[5*t] + 81*Cos[6*t])/
  (23328*a^(4/3)*(2 - 3*Cos[t])^2*Cos[t]^(10/3)*
    (-2 + 3*Cos[t])^(2/3))

kappa2[bow[a_]][t_]:=
(Sqrt[8]*Cos[t]^3*(26 + 3*Cos[2*t] - 6*Cos[4*t] +
Cos[6*t]))/
(a*(18 - 13*Cos[2*t] + 2*Cos[4*t] + Cos[6*t])^(3/2))

kappa2ea[bow[a_]][t_]:=
(8*Cos[t]^6*(-258 + 5066*Cos[2*t] - 2328*Cos[4*t] + 
      325*Cos[6*t] - 22*Cos[8*t] + Cos[10*t]))/
(27*3^(1/3)*a^(4/3)*(26 + 3*Cos[2*t] - 6*Cos[4*t] + Cos[6*t])^(8/3))

kappa2[bowtie[a_,b_]][t_]:=
(a*(11 - 10*b + 12*Cos[2*t] - 12*b*Cos[2*t] + 9*Cos[4*t] + 6*b*Cos[4*t]))/
   (8*((a^2*(5*Cos[t] + 3*Cos[3*t])^2)/16 + 
        ((1 - 2*b + 3*Cos[2*t])^2*Sin[t]^2)/4)^(3/2))

kappa2ea[bowtie[a_,b_]][t_]:=
(2*(1143 - 4300*b - 500*b^2 + 
      16*(101 - 211*b + 110*b^2)*Cos[2*t] + 
      4*(263 + 132*b + 360*b^2)*Cos[4*t] + 240*Cos[6*t] - 
      80*b*Cos[6*t] - 160*b^2*Cos[6*t] + 45*Cos[8*t] + 
      60*b*Cos[8*t] + 20*b^2*Cos[8*t]))/
  (27*3^(1/3)*a^(2/3)*(11 - 10*b - 12*(-1 + b)*Cos[2*t] + 
       3*(3 + 2*b)*Cos[4*t])^(8/3))

kappa2[bulletnose[a_,b_]][t_]:=
(-3*a*b*Cot[t]*Csc[t])/(b^2*Csc[t]^4 + a^2*Sin[t]^2)^(3/2)

kappa2ea[bulletnose[a_,b_]][t_]:=
(-5*(4 + 3*Cos[2*t] + Cos[4*t])*Csc[t]^(10/3))/
  (1458*a^(2/3)*b^(2/3)*Cot[t]^(8/3))

arclength[cardioid[a_]][t_]:=
4*Sqrt[2]*a*Sqrt[1 + Cos[t]]*Tan[t/2]

kappa2[cardioid[a_]][t_]:=
3/(4*Sqrt[2]*a*Sqrt[1 + Cos[t]])

kappa2ea[cardioid[a_]][t_]:=
(11 + 10*Cos[t])/(2916*a^(4/3)*Cos[t/2]^(10/3))

arclength[cardioidunitspeed[a_]][s_]:= s

kappa2[cardioidunitspeed[a_]][s_]:=
3/(a*Sqrt[64 - s^2/a^2])

kappa2[catenary[a_]][t_]:= -Sech[t/a]^2/a

kappa2ea[catenary[a_]][t_]:=
(4 - Cosh[(2*t)/a])/(243*3^(1/3)*a^(4/3)*Cosh[t/a]^(8/3))

kappa2[cayleysextic[a_]][t_]:= Sec[t/2]^2/(3*a)

kappa2ea[cayleysextic[a_]][t_]:=
(5*(8 + 7*Cos[t]))/(5832*3^(2/3)*a^(4/3)*Cos[t/2]^(14/3))

arclength[circle[a_]][t_]:= a*t

kappa2[circle[a_]][t_]:= 1/a

kappa2ea[circle[a_]][t_]:= 1/(27*3^(1/3)*a^(4/3))

arclength[circleinvolute[n_,a_]][t_]:= a*t*Sqrt[t^(2*n)]/n!

kappa2[circleinvolute[n_,a_]][t_]:=n!/(a*t^(n+1))

kappa2ea[circleinvolute[1,a_]][t_]:=
(4 + 9*t^2)/(243*3^(1/3)*a^(4/3)*t^(10/3))

kappa2[circleinvoluteunitspeed[1,a_]][s_]:= 1/Sqrt[2*a*s]

arclength[circleunitspeed[a_]][s_]:= s

kappa2[circleunitspeed[a_]][s_]:= 1/a

kappa2ea[circleunitspeed[a_]][s_]:= 1/(27*3^(1/3)*a^(4/3))

kappa2[cissoid[a_]][t_]:= 3/(a*t*(4 + t^2)^(3/2))

kappa2ea[cissoid[a_]][t_]:=
(1 + 5*t^2 - 5*t^4)/(729*a^(4/3)*t^(10/3))

arclength[clothoid[n_,a_]][t_]:= a*t

kappa2[clothoid[n_,a_]][t_]:= -t^(n +1)/a

kappa2ea[clothoid[1,a_]][t_]:=
(-5 + 9*t^4)/(243*3^(1/3)*a^(4/3)*t^(8/3))

kappa2[cnccoprofile[pm_,a_,b_]][t_]:=
 (-b^3*pm*Cosh[t/a]^2*Sinh[t/a]/
 (a^3*Sqrt[1 - b^2*Sinh[t/a]^2/Sqrt[a^2 - b^2]]) -
  b*Sqrt[a^2 - b^2]*pm*Sinh[t/a]*
       Sqrt[1 - b^2*Sinh[t/a]^2/Sqrt[a^2 - b^2]]/
     a^3)/((b^2*Cosh[t/a]^2)/a^2 + 
     (a^2 - b^2)*(1 - b^2*Sinh[t/a]^2/Sqrt[a^2 - b^2])/a^2)^(3/2)

kappa2ea[cnccoprofile[pm_,a_,b_]][t_]:=
(128*a^4 - 292*a^2*b^2 + 74*b^4 + 
    (32*a^4 + 32*a^2*b^2 + 71*b^4)*Cosh[(2*t)/a] + 
    (-60*a^2*b^2 + 6*b^4)*Cosh[(4*t)/a] + 9*b^4*Cosh[(6*t)/a])/
  (1944*6^(1/3)*a^(4/3)*b^(2/3)*
    (-2*a^2 + b^2 + b^2*Cosh[(2*t)/a])^(5/3)*Sinh[t/a]^(8/3))

kappa2[cnchyprofile[pm_,a_,b_]][t_]:=
(a*(a + 3*b*Sec[t] - 2*b*Sec[t]^3))/
  (a^2 + 2*a*b*Sec[t] + b^2*Sec[t]^4)^(3/2)

kappa2ea[cnchyprofile[pm_,a_,b_]][t_]:=
-(-128*a^4 - 292*a^2*b^2 - 74*b^4 + 
     (32*a^4 - 32*a^2*b^2 + 71*b^4)*Cosh[(2*t)/a] - 
     6*b^2*(10*a^2 + b^2)*Cosh[(4*t)/a] + 9*b^4*Cosh[(6*t)/a])/
  (7776*3^(1/3)*a^(4/3)*b^(2/3)*Cosh[t/a]^(8/3)*
    (a^2 - b^2*Sinh[t/a]^2)^(5/3))

kappa2[cochleoid[n_,a_]][t_]:=
 (Sqrt[2]*t^(1 + n)*Sin[t]^(1 - n)*
    (-n + n^2 + t^2 + 2*n*t^2 + n^2*t^2 + n*Cos[2*t] - 
      n^2*Cos[2*t] - t^2*Cos[2*t] + n^2*t^2*Cos[2*t] - 
      2*n^2*t*Sin[2*t]))/
  (a*(n^2 + t^2 + n^2*t^2 - n^2*Cos[2*t] - t^2*Cos[2*t] + 
       n^2*t^2*Cos[2*t] - 2*n^2*t*Sin[2*t])^(3/2))

kappa2ea[cochleoid[1,a_]][t_]:=
(216*a^2*t + 352*a^2*t^3 + 72*t^4 - 24*t^6 + 576*t^8 + 
    6*t*(96*t^5 + a^2*(9 + 68*t^2))*Cos[2*t] + 
    24*t*(-9*a^2 + 4*a^2*t^2 - 3*t^3 + t^5)*Cos[4*t] - 
    54*a^2*t*Cos[6*t] + 8*a^2*t^3*Cos[6*t] - 
    81*a^2*Sin[2*t] - 612*a^2*t^2*Sin[2*t] - 
    288*t^5*Sin[2*t] - 192*t^7*Sin[2*t] - 
    288*a^2*t^2*Sin[4*t] - 144*t^5*Sin[4*t] + 
    27*a^2*Sin[6*t] - 36*a^2*t^2*Sin[6*t])/
  (972*3^(1/3)*t^6*((a^2*(2*t - Sin[2*t]))/t^3)^(2/3)*
    (-2*t + Sin[2*t])^2)

kappa2[conchoid[a_,b_]][t_]:=
(a*(a + 3*b*Sec[t] - 2*b*Sec[t]^3))/
  (a^2 + 2*a*b*Sec[t] + b^2*Sec[t]^4)^(3/2)

kappa2ea[conchoid[a_,b_]][t_]:=
(35*a^2 - 368*b^2 - 368*a*b*Cos[t] + 
    8*(7*a^2 - 2*b^2)*Cos[2*t] + 28*a^2*Cos[4*t] - 
    80*b^2*Cos[4*t] + 96*a*b*Cos[5*t] + 8*a^2*Cos[6*t] + 
    80*b^2*Cos[6*t] + 16*a*b*Cos[7*t] + a^2*Cos[8*t])/
  (54*2^(2/3)*3^(1/3)*a^(2/3)*
    (-2*b + 3*a*Cos[t] + 6*b*Cos[2*t] + a*Cos[3*t])^(8/3))

kappa2[coshspiral[n_,a_]][t_]:=
(2*Sqrt[2]*(n^2 + 1)*Cosh[n*t]^4)/
   (a*(1 - n^2 + Cosh[2*n*t] + n^2*Cosh[2*n*t])^(3/2))

kappa2ea[coshspiral[n_,a_]][t_]:=
((9 + 41*n^2 + (9 + n^2)*Cosh[2*n*t])*Sech[n*t]^(2/3))/
  (486*3^(1/3)*a^(4/3)*(1 + n^2)^(2/3))

kappa2[cothspiral[a_]][t_]:=
 (-Sin[2*a*t] + a*Sinh[2*t])/Sqrt[1 + a^2]

kappa2ea[cpcprofile[cothspiral[a_]]][t_]:=
-(-27 + 234*a^2 - 27*a^4 + 36*Cos[4*a*t] - 28*a^2*Cos[4*a*t] - 
     9*Cos[8*a*t] - a^2*Cos[8*a*t] - 
     288*a^2*Cos[2*a*t]*Cosh[2*t] - 
     16*a^2*Cos[6*a*t]*Cosh[2*t] - 28*a^2*Cosh[4*t] + 
     36*a^4*Cosh[4*t] + 144*a^2*Cos[4*a*t]*Cosh[4*t] - 
     16*a^2*Cos[2*a*t]*Cosh[6*t] - a^2*Cosh[8*t] - 
     9*a^4*Cosh[8*t] + 240*a*Sin[2*a*t]*Sinh[2*t] - 
     240*a^3*Sin[2*a*t]*Sinh[2*t] - 
     60*a*Sin[6*a*t]*Sinh[2*t] - 12*a^3*Sin[6*a*t]*Sinh[2*t] - 
     24*a*Sin[4*a*t]*Sinh[4*t] + 24*a^3*Sin[4*a*t]*Sinh[4*t] + 
     12*a*Sin[2*a*t]*Sinh[6*t] + 60*a^3*Sin[2*a*t]*Sinh[6*t])/
  (1944*3^(1/3)*(1 + a^2)^(2/3)*(-Sin[2*a*t] + a*Sinh[2*t])^(8/3))

kappa2[cpcprofile[pm_,a_,b_]][t_]:=
(Sqrt[2]*b*pm*Cos[t/a])/(a^2*((2*a^2 - b^2 + b^2*Cos[2*t/a])/a^2)^(1/2))

kappa2ea[cpcprofile[pm_,a_,b_]][t_]:=
(Sec[t/a]^2*(a^6*b*pm*Sin[t/a]^3*
       Sqrt[1 - (b^2*Sin[t/a]^2)/a^2] - 
      b^7*pm*Cos[t/a]^6*Sin[t/a]^3*
       Sqrt[1 - (b^2*Sin[t/a]^2)/a^2] - 
      3*a^4*b^3*pm*Sin[t/a]^5*
       Sqrt[1 - (b^2*Sin[t/a]^2)/a^2] + 
      3*a^2*b^5*pm*Sin[t/a]^7*
       Sqrt[1 - (b^2*Sin[t/a]^2)/a^2] - 
   b^7*pm*Sin[t/a]^9*Sqrt[1 - (b^2*Sin[t/a]^2)/a^2] - 
    3*b^2*Cos[t/a]^4*(-4*a^7 + 9*a^5*b^2*Sin[t/a]^2 + 
         a*b^6*Sin[t/a]^6 - 
         a^2*b^3*pm*Sin[t/a]^3*
          Sqrt[1 - (b^2*Sin[t/a]^2)/a^2] + 
     b^5*pm*Sin[t/a]^5*Sqrt[1 - (b^2*Sin[t/a]^2)/a^2])
        - 3*Cos[t/a]^2*(a^9 - 3*a^5*b^4*Sin[t/a]^4 + 
         5*a^3*b^6*Sin[t/a]^6 - 2*a*b^8*Sin[t/a]^8 + 
         a^4*b^3*pm*Sin[t/a]^3*
          Sqrt[1 - (b^2*Sin[t/a]^2)/a^2] - 
         2*a^2*b^5*pm*Sin[t/a]^5*
          Sqrt[1 - (b^2*Sin[t/a]^2)/a^2] + 
    b^7*pm*Sin[t/a]^7*Sqrt[1 - (b^2*Sin[t/a]^2)/a^2])
        + (3*(2*a^7*b^2*Sin[2*t/a]^2 + 
           3*a^3*b^6*Sin[2*t/a]^4))/8))/
  (243*3^(1/3)*a^3*(a^2 - b^2*Sin[t/a]^2)^4*
    ((b*pm*Cos[t/a]*Sqrt[1 - (b^2*Sin[t/a]^2)/a^2])/
       (a^2 - b^2*Sin[t/a]^2))^(2/3))

kappa2[crosscurve[a_,b_]][t_]:=
(3*a*b*Csc[t]^2*Sec[t]^2)/
 (b^2*Cot[t]^2*Csc[t]^2 + a^2*Sec[t]^2*Tan[t]^2)^(3/2)

kappa2ea[crosscurve[a_,b_]][t_]:=
(-5*(31 + Cos[4*t])*Csc[t]^(2/3)*Sec[t]^(2/3))/
  (5832*a^(2/3)*b^(2/3))

kappa2[cubicparabola[a_,b_,c_,d_]][t_]:=
 (2*b + 6*a*t)/(1 + (c + 2*b*t + 3*a*t^2)^2)^(3/2)

kappa2ea[cubicparabola[a_,b_,c_,d_]][t_]:=
(-5*a^2)/(27*2^(2/3)*3^(1/3)*(b + 3*a*t)^(8/3))

kappa2[cycloid[a_,b_]][t_]:=
b*(-b + a*Cos[t])/(a^2 + b^2 - 2*a*b*Cos[t])^(3/2)

kappa2ea[cycloid[a_,b_]][t_]:=
(-4*a^2 + 9*b^2 - 6*a*b*Cos[t] + a^2*Cos[2*t])/
  (243*3^(1/3)*(b - a*Cos[t])^2*(b*(-b + a*Cos[t]))^(2/3))

kappa2[cycloidunitspeed[a_,a_]][s_]:=
-1/(Sqrt[8*a - s]*Sqrt[s])

kappa2ea[cycloidunitspeed[a_,a_]][s_]:=
(8*(8*a^2 + 8*a*s - s^2))/(243*3^(1/3)*(8*a - s)^(5/3)*s^(5/3))

arclength[delaunay[a_,e_]][s_]:= s

kappa2[delaunay[a_,e_]][s_]:=
e*(e - Cos[s/a])/(a*(1 - 2*e*Cos[s/a] + e^2))

arclength[deltoid[a_]][t_]:=
(-8*a*Cot[3*t/2]*Sqrt[Sin[3*t/2]^2])/3

kappa2[deltoid[a_]][t_]:=-Csc[3*t/2]/(8*a)

kappa2ea[deltoid[a_]][t_]:=
1/(27*6^(1/3)*a^(4/3)*(1 - Cos[3*t])^(5/3))

kappa2[deltoidinvolute[a_]][t_]:=
(-3*Csc[3*t/4]^2)/(16*a)

kappa2ea[deltoidinvolute[a_]][t_]:=
5/(576*2^(1/3)*a^(4/3)*Sin[3*t/4]^(14/3))

arclength[deltoidunitspeed[a_]][s_]:= s

kappa2[deltoidunitspeed[a_]][s_]:= -1/Sqrt[64*a^2 - 9*s^2]

kappa2ea[deltoidunitspeed[a_]][s_]:=
(-256*a^(8/3)*(-64/3 + (3*s^2)/a^2)^(1/3))/(27*(64*a^2 - 9*s^2)^2)

kappa2[diamond[n_,a_,b_]][t_]:=
(a*b*(2 - n)*n^2*Cos[t]^(n - 1)*Sin[t]^(n - 1))/
  (a^2*n^2*Cos[t]^(-2 + 2*n)*Sin[t]^2 + 
     b^2*n^2*Cos[t]^2*Sin[t]^(-2 + 2*n))^(3/2)

kappa2ea[diamond[n_,a_,b_]][t_]:=
((-4 + n)*(14 - 17*n + (2 + n)*Cos[4*t]))/
  (1944*3^(1/3)*a^(2/3)*b^(2/3)*(-2 + n)^(2/3)*n^(4/3)*
    Cos[t]^((2*(2 + n))/3)*Sin[t]^((2*(2 + n))/3))

kappa2[eight][t_]:=
 (-3*Sin[t] + 2*Sin[t]^3)/(1 - 3*Cos[t]^2 + 4*Cos[t]^4)^(3/2)

kappa2ea[eight][t_]:=
(-20*2^(2/3)*Cos[2*t])/(27*3^(1/3)*(-3*Sin[t] - Sin[3*t])^(8/3))

arclength[ellipse[a_,b_]][t_]:= b*EllipticE[t,(b^2 - a^2)/b^2]

kappa2[ellipse[a_,b_]][t_]:=
a*b/(b^2*Cos[t]^2 + a^2*Sin[t]^2)^(3/2)

kappa2ea[ellipse[a_,b_]][t_]:= 1/(27*3^(1/3)*(a*b)^(2/3))

kappa2[ellipsebis[a_,b_]][t_]:=
2*Sqrt[2]*a*b*Cosh[t]^3/(-a^2 + 2*b^2 + a^2*Cosh[2*t])^(3/2)

kappa2ea[ellipsebis[a_,b_]][t_]:= 1/(27*3^(1/3)*(a*b)^(2/3))

arclength[elasticainflect[a_,k_]][s_]:= s

kappa2[elasticainflect[a_,k_]][s_]:= 2*a*k*JacobiCN[a*s,k^2]

arclength[elasticanoninflect[a_,k_]][s_]:= s

kappa2[elasticanoninflect[a_,k_]][s_]:= 2*a*JacobiDN[a*s/k,k^2]/k

arclength[epicycloid[a_,b_]][t_]:=
(-4*b*Cot[a*t/(2*b)]*Sqrt[(a + b)^2*Sin[a*t/(2*b)]^2])/a

kappa2[epicycloid[a_,b_]][t_]:= (a + 2*b)*Csc[a*t/(2*b)]/(4*b*(a + b))

kappa2ea[epicycloid[a_,b_]][t_]:=
-((-4*a^2 - 9*a*b - 9*b^2 + (2*a^2 + 9*a*b + 9*b^2)*Cos[a*t/b])*
     Csc[a*t/(2*b)]^2)/
  (486*3^(1/3)*b^(4/3)*(a + b)^(4/3)*(a + 2*b)^(2/3)*
    (-1 + Cos[a*t/b])^(2/3))

arclength[epicycloidunitspeed[a_,b_]][s_]:= s

kappa2[epicycloidunitspeed[a_,b_]][s_]:=
(a + 2*b)/(Sqrt[a]*Sqrt[s]*Sqrt[8*b*(a + b) - a*s])

kappa2[epispiral[n_,a_]][t_]:=
(-2*Sqrt[2]*(n^2 - 1)*Sin[n*t]^4)/
  (a*(1 + n^2 + (n^2 - 1)*Cos[2*n*t])^(3/2))

kappa2ea[epispiral[n_,a_]][t_]:=
((9 - 41*n^2 + (-9 + n^2)*Cos[2*n*t])*Csc[n*t]^(2/3))/
  (486*3^(1/3)*a^(4/3)*(n^2 - 1)^(2/3))

kappa2[epitrochoid[a_,b_,h_]][t_]:=
(b^3 + a*h^2 + b*h^2 - a*b*h*Cos[a*t/b] -  2*b^2*h*Cos[a*t/b])/
  ((a + b)*(b^2 + h^2 - 2*b*h*Cos[a*t/b])^(3/2))

kappa2ea[epitrochoid[a_,b_,h_]][t_]:=
(18*b^8 - 8*a^4*b^2*h^2 - 5*a^3*b^3*h^2 + 67*a^2*b^4*h^2 + 144*a*b^5*h^2 + 
    72*b^6*h^2 + 18*a^4*h^4 + 72*a^3*b*h^4 + 108*a^2*b^2*h^4 + 72*a*b^3*h^4 + 
    18*b^4*h^4 - 6*b*(a + 2*b)*h*(2*a^3*h^2 + 6*b^3*(b^2 + h^2) - 
       a^2*(b^3 - 11*b*h^2) + 3*a*(b^4 + 5*b^2*h^2))*Cos[a*t/b] + 
    b^2*(a + 2*b)^2*(2*a^2 + 9*a*b + 9*b^2)*h^2*Cos[(2*a*t)/b])/
  (486*3^(1/3)*(a + b)^(4/3)*(b^3 + a*h^2 + b*h^2 - b*(a + 2*b)*
           h*Cos[a*t/b])^(8/3))

kappa2[fermatspiral[a_]][t_]:=
((3*a^2)/(4*t) + a^2*t)/(a^2/(4*t) + a^2*t)^(3/2)

kappa2ea[fermatspiral[a_]][t_]:=
(-45 + 1740*t^2 + 1552*t^4 + 576*t^6)/
  (243*2^(2/3)*3^(1/3)*a^(4/3)*t^(4/3)*(3 + 4*t^2)^(8/3))

kappa2[folium][t_]:=
2*(1 + t^3)^4/(3*(1 + 4*t^2 - 4*t^3 - 4*t^5 + 4*t^6 + t^8)^(3/2))

kappa2ea[folium][t_]:=
(10*2^(1/3)*t)/(81*3^(2/3)*(1 + t^3)^(2/3))

kappa2[folium[n_,a_,b_,c_]][t_]:=
(c^2*n*(1 + n)*t^(-2 + 2*n))/
   ((a + b*t^(2*n + 1))^2*
     ((c^2*t^(2*n)*(a + a*n - b*n*t^(2*n + 1))^2)/
         (a + b*t^(2*n + 1))^4 + 
  (c^2*t^(-2 + 2*n)*(a*n - b*t^(2*n + 1) -
  b*n*t^(2*n + 1))^2)/(a + b*t^(2*n + 1))^4)^(3/2))

kappa2ea[folium[n_,a_,b_,c_]][t_]:=
(a^2*(-2 + n + n^2) + 2*a*b*(8 + 41*n + 41*n^2)*
     t^(1 + 2*n) + b^2*(-2 + n + n^2)*t^(2 + 4*n))/
  (243*3^(1/3)*c^(4/3)*n^(2/3)*(1 + n)^(2/3)*
    t^((2*(1 + 2*n))/3)*(a + b*t^(1 + 2*n))^(2/3))

kappa2[hankelspiral[n_,a_]][t_]:=
(-(a^2*(BesselJ[-2 + n,t] - 2*BesselJ[n,t] +  BesselJ[2 + n,t])*
         (BesselY[-1 + n,t] - BesselY[1 + n,t]))/8 + 
     (a^2*(BesselJ[-1 + n,t] - BesselJ[1 + n,t])*
        (BesselY[-2 + n,t] - 2*BesselY[n,t] +  BesselY[2 + n,t]))/8)/
 ((a^2*(BesselJ[-1 + n,t] - BesselJ[1 + n,t])^2)/4 + 
 (a^2*(BesselY[-1 + n,t] -  BesselY[1 + n,t])^2)/4)^(3/2)

kappa2ea[hankelspiral[n_,a_]][t_]:=
(12*(-((BesselJ[-2 + n, t] - 2*BesselJ[n, t] + 
            BesselJ[2 + n, t])*
          (BesselY[-1 + n, t] - BesselY[1 + n, t])) + 
       BesselJ[-1 + n, t]*
        (BesselY[-2 + n, t] - 2*BesselY[n, t] + 
          BesselY[2 + n, t]) - 
       BesselJ[1 + n, t]*
        (BesselY[-2 + n, t] - 2*BesselY[n, t] + 
          BesselY[2 + n, t]))*
     (BesselJ[2 + n, t]*BesselY[-3 + n, t] - 
       BesselJ[-3 + n, t]*BesselY[-2 + n, t] + 
       3*BesselJ[-1 + n, t]*BesselY[-2 + n, t] - 
       3*BesselJ[1 + n, t]*BesselY[-2 + n, t] + 
       BesselJ[3 + n, t]*BesselY[-2 + n, t] - 
       3*BesselJ[2 + n, t]*BesselY[-1 + n, t] + 
       2*BesselJ[-3 + n, t]*BesselY[n, t] - 
       6*BesselJ[-1 + n, t]*BesselY[n, t] + 
       6*BesselJ[1 + n, t]*BesselY[n, t] - 
       2*BesselJ[3 + n, t]*BesselY[n, t] + 
       3*BesselJ[2 + n, t]*BesselY[1 + n, t] - 
       BesselJ[-3 + n, t]*BesselY[2 + n, t] + 
       3*BesselJ[-1 + n, t]*BesselY[2 + n, t] - 
       3*BesselJ[1 + n, t]*BesselY[2 + n, t] + 
       BesselJ[3 + n, t]*BesselY[2 + n, t] + 
       BesselJ[-2 + n, t]*
        (BesselY[-3 + n, t] - 3*BesselY[-1 + n, t] + 
          3*BesselY[1 + n, t] - BesselY[3 + n, t]) - 
       2*BesselJ[n, t]*
        (BesselY[-3 + n, t] - 3*BesselY[-1 + n, t] + 
          3*BesselY[1 + n, t] - BesselY[3 + n, t]) - 
       BesselJ[2 + n, t]*BesselY[3 + n, t]) - 
    5*(-((BesselJ[-3 + n, t] - BesselJ[3 + n, t])*
           (BesselY[-1 + n, t] - BesselY[1 + n, t])) + 
        BesselJ[-1 + n, t]*
         (BesselY[-3 + n, t] - BesselY[3 + n, t]) + 
        BesselJ[1 + n, t]*
         (-BesselY[-3 + n, t] + BesselY[3 + n, t]))^2 - 
    3*(-((BesselJ[-2 + n, t] - 2*BesselJ[n, t] + 
            BesselJ[2 + n, t])*
          (BesselY[-1 + n, t] - BesselY[1 + n, t])) + 
       BesselJ[-1 + n, t]*
        (BesselY[-2 + n, t] - 2*BesselY[n, t] + 
          BesselY[2 + n, t]) - 
       BesselJ[1 + n, t]*
        (BesselY[-2 + n, t] - 2*BesselY[n, t] + 
          BesselY[2 + n, t]))*
     ((BesselJ[-4 + n, t] - 4*BesselJ[-2 + n, t] + 
          6*BesselJ[n, t] - 4*BesselJ[2 + n, t] + 
          BesselJ[4 + n, t])*
        (BesselY[-1 + n, t] - BesselY[1 + n, t]) - 
       BesselJ[-1 + n, t]*
        (BesselY[-4 + n, t] - 4*BesselY[-2 + n, t] + 
          6*BesselY[n, t] - 4*BesselY[2 + n, t] + 
          BesselY[4 + n, t]) + 
       BesselJ[1 + n, t]*
        (BesselY[-4 + n, t] - 4*BesselY[-2 + n, t] + 
          6*BesselY[n, t] - 4*BesselY[2 + n, t] + 
          BesselY[4 + n, t])))/
  (243*3^(1/3)*a^(4/3)*
    (-((BesselJ[-2 + n, t] - 2*BesselJ[n, t] + 
            BesselJ[2 + n, t])*
          (BesselY[-1 + n, t] - BesselY[1 + n, t])) + 
       BesselJ[-1 + n, t]*
        (BesselY[-2 + n, t] - 2*BesselY[n, t] + 
          BesselY[2 + n, t]) - 
       BesselJ[1 + n, t]*
        (BesselY[-2 + n, t] - 2*BesselY[n, t] + 
          BesselY[2 + n, t]))^(8/3))

arclength[hyperbola[a_,b_]][t_]:=
I*b*EllipticE[I*t,(a^2 + b^2)/b^2]

kappa2[hyperbola[a_,b_]][t_]:=
-a*b/(b^2*Cosh[t]^2 + a^2*Sinh[t]^2)^(3/2)

kappa2ea[hyperbola[a_,b_]][t_]:= -1/(27*3^(1/3)*(a*b)^(2/3))

kappa2[hyperbolabis[a_,b_]][t_]:=
-((a*b*Sec[t]^3)/(b^2*Sec[t]^4 + a^2*Sec[t]^2*Tan[t]^2)^(3/2))

kappa2ea[hyperbolabis[a_,b_]][t_]:= -1/(27*3^(1/3)*(a*b)^(2/3))

kappa2[hyperbolicspiral[a_]][t_]:= t^4/(a*(1 + t^2)^(3/2))

kappa2ea[hyperbolicspiral[a_]][t_]:=
(-20 + 9*t^2)/(243*3^(1/3)*a^(4/3)*t^(2/3))

kappa2[hypocycloid[a_,b_]][t_]:=
 ((a - 2*b)*Csc[a*t/(2*b)])/(4*b*(-a + b))

kappa2ea[hypocycloid[a_,b_]][t_]:=
-((-4*a^2 + 9*a*b - 9*b^2 + 
       (2*a^2 - 9*a*b + 9*b^2)*Cos[a*t/b])*
     Csc[a*t/(2*b)]^2)/
  (486*3^(1/3)*(a - 2*b)^(2/3)*(a - b)^(4/3)*b^(4/3)*
    (-1 + Cos[a*t/b])^(2/3))

kappa2[hypocycloidinvolute[a_,b_]][t_]:=
(a*(a - b)^2*(-a + 2*b)^3*Csc[a*t/(4*b)]^2)/(8*b*(a^2 - 3*a*b + 2*b^2)^3)

kappa2ea[hypocycloidinvolute[a_,b_]][t_]:=
-(a^(4/3)*(-13*a^2 + 36*a*b - 36*b^2 + 
       4*(2*a^2 - 9*a*b + 9*b^2)*Cos[a*t/(2*b)]))/
  (7776*3^(1/3)*(a - 2*b)^2*(a - b)^(4/3)*b^(4/3)*Sin[a*t/(4*b)]^(14/3))

kappa2[hypotrochoid[a_,b_,h_]][t_]:=
(b^3 - a*h^2 + b*h^2 + a*b*h*Cos[a*t/b] -
2*b^2*h*Cos[a*t/b])/((a - b)*(b^2 + h^2 - 2*b*h*Cos[a*t/b])^(3/2))

kappa2ea[hypotrochoid[a_,b_,h_]][t_]:=
(18*b^8 - 8*a^4*b^2*h^2 + 5*a^3*b^3*h^2 + 
    67*a^2*b^4*h^2 - 144*a*b^5*h^2 + 72*b^6*h^2 + 
    18*a^4*h^4 - 72*a^3*b*h^4 + 108*a^2*b^2*h^4 - 
    72*a*b^3*h^4 + 18*b^4*h^4 - 
    6*(a - 2*b)*b*h*(2*a^3*h^2 - 6*b^3*(b^2 + h^2) + 
       a^2*(b^3 - 11*b*h^2) + 3*a*(b^4 + 5*b^2*h^2))*
     Cos[a*t/b] + (a - 2*b)^2*b^2*
     (2*a^2 - 9*a*b + 9*b^2)*h^2*Cos[(2*a*t)/b])/
  (486*3^(1/3)*(a - b)^(4/3)*
    (b^3 - a*h^2 + b*h^2 + (a - 2*b)*b*h*Cos[a*t/b])^(8/3))

kappa2[kampyle[a_]][t_]:=
(Sqrt[2]*Cos[t]^3*(1 - 3*Cos[2*t]))/(a*(5 - 3*Cos[2*t])^(3/2))

kappa2ea[kampyle[a_]][t_]:=
-(a^(4/3)*(-13*a^2 + 36*a*b - 36*b^2 + 
       4*(2*a^2 - 9*a*b + 9*b^2)*Cos[a*t/(2*b)]))/
  (7776*3^(1/3)*(a - 2*b)^2*(a - b)^(4/3)*b^(4/3)*Sin[a*t/(4*b)]^(14/3))

kappa2[kappacurve[a_]][t_]:=
(8*Sqrt[2]*(5 + Cos[2*t])*Sin[t]^4)/(a*(9 - Cos[4*t])^(3/2))

kappa2ea[kappacurve[a_]][t_]:=
-((166 + 447*Cos[2*t] + 26*Cos[4*t] + Cos[6*t])*Csc[t]^(2/3))/
  (108*6^(1/3)*a^(4/3)*(5 + Cos[2*t])^(8/3))

kappa2[keplerorbit[a_,e_]][t]:=
(1 - e*Cos[t])^3/(a*(1 + e^2 - 2*e*Cos[t])^(3/2))

kappa2ea[keplerorbit[a_,e_]][t]:= (e^2 - 1)/(27*3^(1/3)*a^(4/3))

kappa2[lehr[a_,b_,c_]][s_]:= a + b*Cos[c*s]

arclength[lemniscate[a_]][t_]:= a*EllipticF[t,-1]

kappa2[lemniscate[a_]][t_]:= 3*Cos[t]/(a*Sqrt[2 - Cos[t]^2])

kappa2ea[lemniscate[a_]][t_]:=
(5*(29 + 76*Cos[2*t] + 15*Cos[4*t]))/
  (2916*2^(1/3)*a^(4/3)*Cos[t]^(8/3)*(-3 + Cos[2*t])^(2/3))

arclength[lemniscateunitspeed[a_]][s_]:= s

kappa2[lemniscateunitspeed[a_]][s_]:=
3*JacobiCN[s/a,-1]/(a*Sqrt[2 - JacobiCN[s/a,-1]^2])

kappa2[limacon[a_,b_]][t_]:=
(8*a^2 + b^2 + 6*a*b*Cos[t])/(4*a^2 + b^2 + 4*a*b*Cos[t])^(3/2)

kappa2ea[limacon[a_,b_]][t_]:=
(256*a^4 + 60*a^2*b^2 + b^4 + 
    16*a*b*(17*a^2 + b^2)*Cos[t] + 40*a^2*b^2*Cos[2*t])/
  (27*3^(1/3)*(8*a^2 + b^2 + 6*a*b*Cos[t])^(8/3))

kappa2[line[a1_,b1_,a2_,b2_]][t_]:=0

kappa2[lissajous[n_,d_,a_,b_]][t_]:=
 (-(a*b*n*Cos[d + n*t]*Sin[t]) + 
    a*b*n^2*Cos[t]*Sin[d + n*t])/
  (b^2*Cos[t]^2 + a^2*n^2*Cos[d + n*t]^2)^(3/2)

kappa2ea[lissajous[n_,d_,a_,b_]][t_]:=
-(2*Cos[t]^2*(5 - 22*n^2 + 8*n^4 + 
        (5 + 2*n^2 + 2*n^4)*Cos[2*(d + n*t)]) + 
     3*(-4*(-1 + 4*n^2)*Cos[d + n*t]^2*Sin[t]^2 + 
        3*n*(1 + n^2)*Sin[2*t]*Sin[2*(d + n*t)]))/
  (972*3^(1/3)*a^(2/3)*b^(2/3)*n^(2/3)*
    (-(Cos[d + n*t]*Sin[t]) + n*Cos[t]*Sin[d + n*t])^(8/3))

kappa2[lituus[a_]][t_]:=
(2*t^(3/2)*(-1 + 4*t^2))/(a*(1 + 4*t^2)^(3/2))

kappa2ea[lituus[a_]][t_]:=
(-9 - 84*t^2 - 944*t^4 + 576*t^6)/
  (243*2^(2/3)*3^(1/3)*a^(4/3)*(-1 + 4*t^2)^(8/3))

kappa2[logspiral[a_,b_]][t_]:=
 1/(a*(1 + b^2)^(1/2)*E^(b*t))

kappa2ea[logspiral[a_,b_]][t_]:=
(9 + b^2)/(243*3^(1/3)*a^(4/3)*(1 + b^2)^(2/3)*E^((4*b*t)/3))

kappa2[logistic[k_,a_,b_]][t_]:=
(b^2*E^(a + b*t)*(-1 + E^(a + b*t))*k)/
((1 + E^(a + b*t))^3*(1 + (b^2*E^(2*(a + b*t))*k^2)/(1 + E^(a + b*t))^4)^(3/2))

kappa2ea[logistic[k_,a_,b_]][t_]:=
(-2*b^(2/3)*(1 - 2*E^(a + b*t) + 12*E^(2*(a + b*t)) - 
      2*E^(3*(a + b*t)) + E^(4*(a + b*t))))/
  (243*3^(1/3)*E^((2*(a + b*t))/3)*(-1 + E^(a + b*t))^(8/3)*k^(2/3))

kappa2[nephroid[a_]][t_]:=Csc[t]/(3*a)

kappa2ea[nephroid[a_]][t_]:=
(43 - 35*Cos[2*t])/(5832*3^(2/3)*a^(4/3)*Sin[t]^(10/3))

kappa2[nielsenspiral[a_]][t_]:=t/a

kappa2[parabola[a_]][t_]:= 1/(2*a*(1 + t^2)^(3/2))

kappa2ea[parabola[a_]][t_]:=0

kappa2[parabolicspiral[n_,a_,b_]][t_]:=
(t^(2*(-1/2 - 2*n) + 6*n)*(2*a^2*t - 6*a^2*n*t + 4*a^2*n^2*t + a^2*t^3 + 
       2*a*b*n*t^(2*n) - 4*a*b*n^2*t^(2*n) + 
       2*a*b*t^(2 + 2*n) + b^2*t^(1 + 4*n)))/
   (a^2 - 4*a^2*n + 4*a^2*n^2 + a^2*t^2 + b^2*t^(4*n) + 
      2*a*b*t^(2*n + 1))^(3/2)

kappa2ea[parabolicspiral[n_,a_,b_]][t_]:=
(t^((4*(-1 + 2*n))/3)*(9*b^4*t^(4 + 8*n) + 
      6*a*b^3*t^(1 + 6*n)*
       (-8*n^3 - 8*n^4 + 6*t^4 + n^2*(2 - 26*t^2) + 
         n*(2 + 13*t^2)) + 
      2*a^2*b^2*t^(4*n)*
       (48*n^5 - 64*n^6 + n*t^2*(26 + t^2) + 
         t^4*(29 + 27*t^2) + 8*n^4*(1 + 65*t^2) - 
         4*n^3*(3 + 158*t^2) + n^2*(2 + 134*t^2 - 118*t^4)) +
         2*a^3*b*t^(1 + 2*n)*
       (480*n^5 - 128*n^6 + 58*t^4 + 18*t^6 - 
         328*n^4*(2 + t^2) + 8*n^3*(51 + 121*t^2) + 
         n*(12 + 214*t^2 - 115*t^4) - 
         2*n^2*(58 + 415*t^2 + t^4)) + 
      a^4*t^2*(-288*n^5 + 64*n^6 + 16*n^4*(31 + 11*t^2) - 
         8*n^3*(51 + 86*t^2) + 
         t^2*(120 + 58*t^2 + 9*t^4) + 
         4*n^2*(40 + 241*t^2 + 19*t^4) - 
         2*n*(12 + 286*t^2 + 77*t^4))))/
  (243*3^(1/3)*(b^2*t^(1 + 4*n) + 
       2*a*b*t^(2*n)*(n - 2*n^2 + t^2) + 
       a^2*t*(2 - 6*n + 4*n^2 + t^2))^(8/3))

arclength[pseudocatenary[a_,b_]][s_]:= s

kappa2[pseudocatenary[a_,b_]][s_]:= -a/(a*b + s^2)

kappa2[piriform[a_,b_]][t_]:=
(-a*b - 3*a*b*Sin[t] + 2*a*b*Sin[t]^3)/
  ((Cos[t/2] + Sin[t/2])^2*
     (a^2 + b^2 - a^2*Sin[t] - b^2*Sin[3*t]))^(3/2)

kappa2ea[piriform[a_,b_]][t_]:=
(8*2^(2/3)*(1 + Sin[t])*(-2*Cos[2*t] + Sin[t]))/
  (27*3^(1/3)*a^(2/3)*b^(2/3)*
    (2 + 3*Sin[t] + Sin[3*t])^(8/3))

kappa2[rose[n_,a_]][t_]:=
 Sqrt[2]*(1 + 3*n^2 + Cos[2*n*t] - n^2*Cos[2*n*t])/
  (a*(1 + n^2 + Cos[2*n*t] - n^2*Cos[2*n*t])^(3/2))

kappa2ea[rose[n_,a_]][t_]:=
(27 + 175*n^2 + 721*n^4 + 229*n^6 + 
    12*(3 + 13*n^2 - 19*n^4 + 3*n^6)*Cos[2*n*t] - 
    (-9 + n^2)*(n^2 - 1)^2*Cos[4*n*t])/
  (243*6^(1/3)*a^(4/3)*(1 + 3*n^2 - 
       (n^2 - 1)*Cos[2*n*t])^(8/3))

arclength[sc][t_]:=
((8/27 + (2*t^2)/3)*Sqrt[t^2*(4 + 9*t^2)])/(2*t)

kappa2[sc][t_]:=6*t^2/(4*t^2 + 9*t^4)^(3/2)

kappa2ea[sc][t_]:= 2*2^(1/3)/(729*t^(10/3))

arclength[scunitspeed][s_]:= s

kappa2[scunitspeed][s_]:=
18/((8 + 27*s)*Sqrt[-4 + (8 + 27*s)^(2/3)])

kappa2ea[scunitspeed][s_]:=
(2*2^(1/3))/(9*3^(2/3)*(-4 + (8 + 27*t)^(2/3))^(5/3))

kappa2[serpentine[a_,b_]][t_]:=
(2*a*b*t*(-3 + b*t^2))/
((1 + b*t^2)^3*(1 + (a^2*(1 - b*t^2)^2)/(1 + b*t^2)^4)^(3/2))

kappa2ea[serpentine[a_,b_]][t_]:=
-(5 + 50*b^2*t^4 - 8*b^3*t^6 + b^4*t^8)/
  (27*2^(2/3)*3^(1/3)*a^(2/3)*b^(2/3)*t^(8/3)*(-3 + b*t^2)^(8/3))

kappa2[sinhspiral[n_,a_]][t_]:= 
(2*Sqrt[2]*(n^2 + 1)*Sinh[n*t]^4)/(a*(-1 + n^2 + (n^2 + 1)*Cosh[2*n*t])^(3/2))

kappa2ea[sinhspiral[n_,a_]][t_]:= 
((-9 - 41*n^2 + (9 + n^2)*Cosh[2*n*t])*Csch[n*t]^(2/3))/
  (486*3^(1/3)*a^(4/3)*(1 + n^2)^(2/3))

kappa2[spring[a_,b_,v_]][t_]:=
a*(2*a*Cos[v] - b*Sin[t - v] + b*Sin[t + v])/
(2*(a^2*Sin[t]^2 + (a*Cos[t]*Cos[v] + b*Sin[v])^2)^(3/2))

kappa2ea[spring[a_,b_,v_]][t_]:=
(18*a^2 - 8*b^2 + 2*b^2*Cos[2*t] - b^2*Cos[2*(t - v)] + 
    18*a^2*Cos[2*v] + 8*b^2*Cos[2*v] - 
    b^2*Cos[2*(t + v)] - 6*a*b*Sin[t - 2*v] + 
    6*a*b*Sin[t + 2*v])/
  (972*3^(1/3)*a^(2/3)*(a*Cos[v] + b*Cos[t]*Sin[v])^(8/3))

kappa2[strophoid[a_]][t_]:=
4*(1 + 3*t^2)/(a*(1 + 6*t^2 + t^4)^(3/2))

kappa2ea[strophoid[a_]][t_]:= 
(2^(2/3)*(3 + 9*t^2 + 5*t^4 - 5*t^6))/(27*3^(1/3)*a^(4/3)*(1 + 3*t^2)^(8/3))

kappa2[talbot[a_,b_,f_]][t_]:=
(4*Sqrt[2]*a^2*b^2)/
  ((a^2 + b^2 + (a^2 - b^2)*Cos[2*t])^(3/2)*(-2*a^2 + f^2 + 3*f^2*Cos[2*t]))

kappa2ea[talbot[a_,b_,f_]][t_]:=
((2/3)^(1/3)*a^(2/3)*b^(2/3)*(8*a^4 - 8*a^2*f^2 + 39*f^4 + 
      20*f^2*(-2*a^2 + f^2)*Cos[2*t] + 5*f^4*Cos[4*t]))/
  (27*(-2*a^2 + f^2 + 3*f^2*Cos[2*t])^(10/3))

kappa2[tanhspiral[a_]][t_]:=
(-Sin[2*a*t] - a*Sinh[2*t])/Sqrt[1 + a^2]

kappa2ea[tanhspiral[a_]][t_]:= 
-(-27 + 234*a^2 - 27*a^4 + 36*Cos[4*a*t] - 
     28*a^2*Cos[4*a*t] - 9*Cos[8*a*t] - a^2*Cos[8*a*t] + 
     288*a^2*Cos[2*a*t]*Cosh[2*t] + 
     16*a^2*Cos[6*a*t]*Cosh[2*t] - 28*a^2*Cosh[4*t] + 
     36*a^4*Cosh[4*t] + 144*a^2*Cos[4*a*t]*Cosh[4*t] + 
     16*a^2*Cos[2*a*t]*Cosh[6*t] - a^2*Cosh[8*t] - 
     9*a^4*Cosh[8*t] - 240*a*Sin[2*a*t]*Sinh[2*t] + 
     240*a^3*Sin[2*a*t]*Sinh[2*t] + 60*a*Sin[6*a*t]*Sinh[2*t] + 
     12*a^3*Sin[6*a*t]*Sinh[2*t] - 24*a*Sin[4*a*t]*Sinh[4*t] + 
     24*a^3*Sin[4*a*t]*Sinh[4*t] - 12*a*Sin[2*a*t]*Sinh[6*t] - 
     60*a^3*Sin[2*a*t]*Sinh[6*t])/
    (1944*3^(1/3)*(1 + a^2)^(2/3)*(Sin[2*a*t] + a*Sinh[2*t])^(8/3))

kappa2[teardrop[a_,b_,c_,d_,n_]][t_]:=
(a*d*n*(b + 2*c*Cos[t])^(n - 2)*Sin[t]^n*
    ((b + 2*c*Cos[t])*(b*Cos[t] + 2*c*Cos[2*t]) + 
      Cot[t]*Csc[t]*((n - 1)* (b*Cos[t] + 2*c*Cos[2*t])^2 - 
         (b + 2*c*Cos[t])*(b + 8*c*Cos[t])*Sin[t]^2)))/
     (a^2*Cos[t]^2 + d^2*n^2*(b + 2*c*Cos[t])^(2*(n - 1))*
     (b*Cos[t] + 2*c*Cos[2*t])^2*Sin[t]^(2*(n - 1)))^(3/2)

kappa2ea[teardrop[a_,b_,c_,d_,n_]][t_]:=
-(70*b^6 - 9936*b^4*c^2 - 33888*b^2*c^4 - 8576*c^6 - 
     315*b^6*n - 12024*b^4*c^2*n - 38448*b^2*c^4*n - 
     11520*c^6*n + 490*b^6*n^2 + 28416*b^4*c^2*n^2 + 
     106128*b^2*c^4*n^2 + 31360*c^6*n^2 - 315*b^6*n^3 - 
     15336*b^4*c^2*n^3 - 52992*b^2*c^4*n^3 - 
     13824*c^6*n^3 + 70*b^6*n^4 + 3120*b^4*c^2*n^4 + 
     10560*b^2*c^4*n^4 + 2560*c^6*n^4 + 
     16*b*c*(3*b^4*(-12 - 111*n + 230*n^2 - 135*n^3 + 
           28*n^4) + 24*c^4*
         (-121 - 123*n + 408*n^2 - 204*n^3 + 40*n^4) + 
        2*b^2*c^2*(-1055 - 1575*n + 4294*n^2 - 2304*n^3 + 
           460*n^4))*Cos[t] + 
     4*(28*b^6 + 2826*b^4*c^2 - 2568*b^2*c^4 - 2282*c^6 - 
        126*b^6*n - 6363*b^4*c^2*n - 16200*b^2*c^4*n - 
        1782*c^6*n + 196*b^6*n^2 + 11346*b^4*c^2*n^2 + 
        42336*b^2*c^4*n^2 + 9064*c^6*n^2 - 126*b^6*n^3 - 
        6669*b^4*c^2*n^3 - 24048*b^2*c^4*n^3 - 
        5400*c^6*n^3 + 28*b^6*n^4 + 1380*b^4*c^2*n^4 + 
        4800*b^2*c^4*n^4 + 1120*c^6*n^4)*Cos[2*t] + 
     3528*b^5*c*Cos[3*t] + 53248*b^3*c^3*Cos[3*t] + 
     31176*b*c^5*Cos[3*t] - 6876*b^5*c*n*Cos[3*t] - 
     85248*b^3*c^3*n*Cos[3*t] - 70920*b*c^5*n*Cos[3*t] + 
     6840*b^5*c*n^2*Cos[3*t] + 
     94144*b^3*c^3*n^2*Cos[3*t] + 
     102624*b*c^5*n^2*Cos[3*t] - 
     4500*b^5*c*n^3*Cos[3*t] - 
     57024*b^3*c^3*n^3*Cos[3*t] - 
     64800*b*c^5*n^3*Cos[3*t] + 1008*b^5*c*n^4*Cos[3*t] + 
     12160*b^3*c^3*n^4*Cos[3*t] + 
     13440*b*c^5*n^4*Cos[3*t] + 56*b^6*Cos[4*t] + 
     15264*b^4*c^2*Cos[4*t] + 99624*b^2*c^4*Cos[4*t] + 
     22464*c^6*Cos[4*t] - 252*b^6*n*Cos[4*t] - 
     31104*b^4*c^2*n*Cos[4*t] - 
     154656*b^2*c^4*n*Cos[4*t] - 36864*c^6*n*Cos[4*t] + 
     392*b^6*n^2*Cos[4*t] + 23232*b^4*c^2*n^2*Cos[4*t] + 
     108120*b^2*c^4*n^2*Cos[4*t] + 
     28992*c^6*n^2*Cos[4*t] - 252*b^6*n^3*Cos[4*t] - 
     16992*b^4*c^2*n^3*Cos[4*t] - 
     67968*b^2*c^4*n^3*Cos[4*t] - 
     18432*c^6*n^3*Cos[4*t] + 56*b^6*n^4*Cos[4*t] + 
     3840*b^4*c^2*n^4*Cos[4*t] + 
     14880*b^2*c^4*n^4*Cos[4*t] + 3840*c^6*n^4*Cos[4*t] + 
     24*b^5*c*Cos[5*t] + 28088*b^3*c^3*Cos[5*t] + 
     76152*b*c^5*Cos[5*t] - 1188*b^5*c*n*Cos[5*t] - 
     57960*b^3*c^3*n*Cos[5*t] - 122040*b*c^5*n*Cos[5*t] + 
     2760*b^5*c*n^2*Cos[5*t] + 
     38864*b^3*c^3*n^2*Cos[5*t] + 
     68256*b*c^5*n^2*Cos[5*t] - 2124*b^5*c*n^3*Cos[5*t] - 
     34272*b^3*c^3*n^3*Cos[5*t] - 
     43488*b*c^5*n^3*Cos[5*t] + 528*b^5*c*n^4*Cos[5*t] + 
     8000*b^3*c^3*n^4*Cos[5*t] + 
     9600*b*c^5*n^4*Cos[5*t] + 16*b^6*Cos[6*t] - 
     1332*b^4*c^2*Cos[6*t] + 16464*b^2*c^4*Cos[6*t] + 
     18312*c^6*Cos[6*t] - 72*b^6*n*Cos[6*t] + 
     234*b^4*c^2*n*Cos[6*t] - 36144*b^2*c^4*n*Cos[6*t] - 
     30888*c^6*n*Cos[6*t] + 112*b^6*n^2*Cos[6*t] + 
     8004*b^4*c^2*n^2*Cos[6*t] + 
     31104*b^2*c^4*n^2*Cos[6*t] + 
     17376*c^6*n^2*Cos[6*t] - 72*b^6*n^3*Cos[6*t] - 
     7506*b^4*c^2*n^3*Cos[6*t] - 
     38304*b^2*c^4*n^3*Cos[6*t] - 
     11808*c^6*n^3*Cos[6*t] + 16*b^6*n^4*Cos[6*t] + 
     2040*b^4*c^2*n^4*Cos[6*t] + 
     9600*b^2*c^4*n^4*Cos[6*t] + 2688*c^6*n^4*Cos[6*t] + 
     84*b^5*c*Cos[7*t] - 6472*b^3*c^3*Cos[7*t] - 
     5568*b*c^5*Cos[7*t] - 378*b^5*c*n*Cos[7*t] + 
     10008*b^3*c^3*n*Cos[7*t] + 8640*b*c^5*n*Cos[7*t] + 
     756*b^5*c*n^2*Cos[7*t] + 
     12464*b^3*c^3*n^2*Cos[7*t] + 
     6912*b*c^5*n^2*Cos[7*t] - 630*b^5*c*n^3*Cos[7*t] - 
     14400*b^3*c^3*n^3*Cos[7*t] - 
     21888*b*c^5*n^3*Cos[7*t] + 168*b^5*c*n^4*Cos[7*t] + 
     4160*b^3*c^3*n^4*Cos[7*t] + 
     6144*b*c^5*n^4*Cos[7*t] + 2*b^6*Cos[8*t] + 
     48*b^4*c^2*Cos[8*t] - 10464*b^2*c^4*Cos[8*t] - 
     5760*c^6*Cos[8*t] - 9*b^6*n*Cos[8*t] - 
     648*b^4*c^2*n*Cos[8*t] + 18288*b^2*c^4*n*Cos[8*t] + 
     11520*c^6*n*Cos[8*t] + 14*b^6*n^2*Cos[8*t] + 
     2112*b^4*c^2*n^2*Cos[8*t] + 
     12144*b^2*c^4*n^2*Cos[8*t] - 2688*c^6*n^2*Cos[8*t] - 
     9*b^6*n^3*Cos[8*t] - 2232*b^4*c^2*n^3*Cos[8*t] - 
     16128*b^2*c^4*n^3*Cos[8*t] - 4608*c^6*n^3*Cos[8*t] + 
     2*b^6*n^4*Cos[8*t] + 720*b^4*c^2*n^4*Cos[8*t] + 
     4800*b^2*c^4*n^4*Cos[8*t] + 1536*c^6*n^4*Cos[8*t] + 
     12*b^5*c*Cos[9*t] - 152*b^3*c^3*Cos[9*t] - 
     6336*b*c^5*Cos[9*t] - 54*b^5*c*n*Cos[9*t] - 
     504*b^3*c^3*n*Cos[9*t] + 10944*b*c^5*n*Cos[9*t] + 
     108*b^5*c*n^2*Cos[9*t] + 3088*b^3*c^3*n^2*Cos[9*t] + 
     8448*b*c^5*n^2*Cos[9*t] - 90*b^5*c*n^3*Cos[9*t] - 
     4032*b^3*c^3*n^3*Cos[9*t] - 
     10368*b*c^5*n^3*Cos[9*t] + 24*b^5*c*n^4*Cos[9*t] + 
     1600*b^3*c^3*n^4*Cos[9*t] + 
     3072*b*c^5*n^4*Cos[9*t] + 12*b^4*c^2*Cos[10*t] - 
     48*b^2*c^4*Cos[10*t] - 1000*c^6*Cos[10*t] - 
     126*b^4*c^2*n*Cos[10*t] - 432*b^2*c^4*n*Cos[10*t] + 
     1224*c^6*n*Cos[10*t] + 372*b^4*c^2*n^2*Cos[10*t] + 
     2304*b^2*c^4*n^2*Cos[10*t] + 
     3488*c^6*n^2*Cos[10*t] - 378*b^4*c^2*n^3*Cos[10*t] - 
     3744*b^2*c^4*n^3*Cos[10*t] - 
     3168*c^6*n^3*Cos[10*t] + 120*b^4*c^2*n^4*Cos[10*t] + 
     1920*b^2*c^4*n^4*Cos[10*t] + 896*c^6*n^4*Cos[10*t] + 
     8*b^3*c^3*Cos[11*t] + 168*b*c^5*Cos[11*t] - 
     216*b^3*c^3*n*Cos[11*t] - 360*b*c^5*n*Cos[11*t] + 
     752*b^3*c^3*n^2*Cos[11*t] + 
     480*b*c^5*n^2*Cos[11*t] - 
     864*b^3*c^3*n^3*Cos[11*t] - 
     1440*b*c^5*n^3*Cos[11*t] + 
     320*b^3*c^3*n^4*Cos[11*t] + 
     1152*b*c^5*n^4*Cos[11*t] + 24*b^2*c^4*Cos[12*t] + 
     64*c^6*Cos[12*t] - 288*b^2*c^4*n*Cos[12*t] + 
     936*b^2*c^4*n^2*Cos[12*t] - 320*c^6*n^2*Cos[12*t] - 
     1152*b^2*c^4*n^3*Cos[12*t] + 
     480*b^2*c^4*n^4*Cos[12*t] + 256*c^6*n^4*Cos[12*t] + 
     24*b*c^5*Cos[13*t] - 216*b*c^5*n*Cos[13*t] + 
     672*b*c^5*n^2*Cos[13*t] - 864*b*c^5*n^3*Cos[13*t] + 
     384*b*c^5*n^4*Cos[13*t] + 8*c^6*Cos[14*t] - 
     72*c^6*n*Cos[14*t] + 224*c^6*n^2*Cos[14*t] - 
     288*c^6*n^3*Cos[14*t] + 128*c^6*n^4*Cos[14*t])/
  (31104*3^(1/3)*a^(2/3)*d^(2/3)*n^(2/3)*
    (b + 2*c*Cos[t])^((2*(1 + n))/3)*
    Sin[t]^((2*(1 + n))/3)*
    (b^2*(-1 + n)*Cos[t]^3 + 
       4*b*c*(-1 + n)*Cos[t]^2*Cos[2*t] + 
       2*c^2*Cos[t]*(-2 + n + n*Cos[4*t]) + 
       2*c*(-2*b + c*Cos[t] - b*Cos[2*t] + c*Cos[3*t])*
        Sin[t]^2)^(8/3))

arclength[tractrix[a_]][t_]:= a*Log[Sin[t]]

kappa2[tractrix[a_]][t_]:= -Sqrt[Tan[t]^2/a^2]

kappa2ea[tractrix[a_]][t_]:=
((17 - 24*Cos[2*t] - 9*Cos[4*t])*Csc[t/2]^(2/3)*Sec[t/2]^(2/3))/
  (1944*2^(2/3)*3^(1/3)*a^(4/3)*Cos[t]^(10/3))

kappa2[tractrixminus[a_]][s]:=
(2*a/s^2 - s^2/(a^2 - s^2)^(3/2) - (a^2 - s^2)^(-1/2) + a^3/(a^2*s^2 - s^4) + 
     (a^2*(-2*a^3 + 3*a*s^2 - 2*(a^2 - s^2)^(3/2)))/
  (s^2*(a^2 - s^2)^(3/2)*(a + (a^2 - s^2)^(1/2))))/
   (1 + (a^2 - s^2 - 2*a*(a^2 - s^2)^(1/2))^2/(a^2*s^2 - s^4))^(3/2)

kappa2ea[tractrixminus[a_]][s]:=
-((-a^2*s^2 + s^4)^(2/3)*
     (64*a^13 - 32*a^11*s^2 - 20*a^9*s^4 - 290*a^7*s^6 + 
       478*a^5*s^8 - 236*a^3*s^10 + 36*a*s^12 + 
       64*a^12*Sqrt[a^2 - s^2] - 12*a^8*s^4*Sqrt[a^2 - s^2] - 
       292*a^6*s^6*Sqrt[a^2 - s^2] + 333*a^4*s^8*Sqrt[a^2 - s^2] - 
       105*a^2*s^10*Sqrt[a^2 - s^2] + 
       8*s^12*Sqrt[a^2 - s^2]))/
  (243*3^(1/3)*a^(2/3)*s^2*(a^2 - s^2)^(3/2)*(a + Sqrt[a^2 - s^2])^6*
    (-2*a^2 + 2*s^2 + a*Sqrt[a^2 - s^2])^(8/3))

kappa2[tractrixplus[a_]][s_]:=
((-2*a)/s^2 + s^2/(a^2 - s^2)^(3/2) + 
     (a^2 - s^2)^(-1/2) - a^3/(a^2*s^2 - s^4) + 
     (a^2*(2*a^3 - 3*a*s^2 + 2*(a^2 - s^2)^(3/2)))/
  (s^2*(a^2 - s^2)^(3/2)*(a + (a^2 - s^2)^(1/2))))/
   (1 + (-a^2 + s^2 + 2*a*(a^2 - s^2)^(1/2))^2/(a^2*s^2 - s^4))^(3/2)

kappa2ea[tractrixplus[a_]][s_]:=
-(64*a^13 - 32*a^11*s^2 - 20*a^9*s^4 - 290*a^7*s^6 + 
     478*a^5*s^8 - 236*a^3*s^10 + 36*a*s^12 + 
     64*a^12*Sqrt[a^2 - s^2] - 12*a^8*s^4*Sqrt[a^2 - s^2] - 
     292*a^6*s^6*Sqrt[a^2 - s^2] + 333*a^4*s^8*Sqrt[a^2 - s^2] - 
     105*a^2*s^10*Sqrt[a^2 - s^2] + 8*s^12*Sqrt[a^2 - s^2])/
    (243*3^(1/3)*a^(2/3)*s^(2/3)*(a^2 - s^2)^(5/6)*
    (a + Sqrt[a^2 - s^2])^6*(-2*a^2 + 2*s^2 + a*Sqrt[a^2 - s^2])^(8/3))

arclength[tractrixunitspeed[a_]][s_]:= s

kappa2[tractrixunitspeed[a_]][s_]:=
-1/(a*E^(s/a)*Sqrt[1 - E^(-2*s/a)])

kappa2ea[tractrixunitspeed[a_]][t_]:=
-(9 - 15*E^((2*t)/a) + 2*E^(4*t/a))/
  (243*3^(1/3)*a^(4/3)*(-1 + E^(2*t/a))^(5/3))

kappa2[trisectrix[a_]][t_]:=
(24*Cos[t]^4)/(a*(1 + 8*Cos[t]^2)^(3/2))

kappa2ea[trisectrix[a_]][t_]:=
(5*(1 + 2*Cos[2*t])*Sec[t]^(2/3))/(729*a^(4/3))

kappa2[tschirnhausen[n_,a_]][t_]:=
3*Sqrt[2]*Sqrt[-((Cos[t/3]^(2*(1 + n))*
        (9 - 2*n + n^2 - (n^2 - 9)*Cos[2*t/3])^2)/
      (a^2*(-9 - n^2 + (n^2 - 9)*Cos[2*t/3])^3))]

kappa2ea[tschirnhausen[n_,a_]][t_]:=
-(Cos[t/3]^((2*(-1 + 2*n))/3)*
   (-65610 + 39204*n - 11062*n^2 + 1848*n^3 - 246*n^4 + 36*n^5 - 10*n^6 + 
       (-98415 + 51408*n - 9875*n^2 + 480*n^3 + 115*n^4 - 
          48*n^5 + 15*n^6)*Cos[2*t/3] + 
       (-39366 + 12204*n + 2790*n^2 - 1464*n^3 + 
          230*n^4 + 12*n^5 - 6*n^6)*Cos[4*t/3] - 
       6561*Cos[2*t] + 1539*n^2*Cos[2*t] - 
       99*n^4*Cos[2*t] + n^6*Cos[2*t]))/
  (2916*2^(1/3)*a^(4/3)*
    (-9 + 2*n - n^2 + (-9 + n^2)*Cos[(2*t)/3])^(8/3))

kappa2[zcprofile[pm_,a_,c_]][t_]:=
(c^4/(a^3*(1 - c^2/(a^2*t^2))^(3/2)*t^5) + 
    (2*c^2)/(a*(1 - c^2/(a^2*t^2))^(1/2)*t^3) - 
    a^4*pm*t^2/(-c^2 + a^2*t^2)^(3/2) + 
    (a^2*pm)/(-c^2 + a^2*t^2)^(1/2))/
  (1 + (-(c^2/(a*(1 - c^2/(a^2*t^2))^(1/2)*t^2)) + 
        a^2*pm*t/(-c^2 + a^2*t^2)^(1/2))^2)^(3/2)

kappa2ea[zcprofile[pm_,a_,c_]][t_]:=
-((c^2*t - a^2*t^3)^(1/3)*
     (2*c^6 + a^2*c^4*(1 - 18*pm)*t^2 - 
       24*a^4*c^2*(-2 + pm)*t^4 - 9*a^6*(-2 + pm)^2*t^6))/
  (243*c^(4/3)*t*(-3*c^2 + 3*a^2*t^2)^(1/3)*
    (c^2 + a^2*(-2 + pm)*t^2)^(8/3))

End[]

EndPackage[]
