(*:Date: Tue Feb  3 04:42:33 EST 1998
*)

(*:Title: KAPPATAU.m *)

(*:Author: Alfred Gray *)

(* Copyright 1996-1998 by Alfred Gray *)

(*:Package Version: 1.0 *)

(*:Mathematica Version: 2.2, 3.0 *)


(*:Summary: 
This package consists of formulas for the  curvature, torsion and tangent developables of many space curves.*)

BeginPackage["KAPPATAU`","CURVES`","CSPROGS`"]

    kappa::usage="t->kappa[alpha][t] is the unsigned curvature of the curve alpha:(a,b)->R^3."

    tau::usage="t->tau[alpha][t] is the torsion of the curve alpha:(a,b)->R^3."

    tandev[alpha_][u_,v_]:= alpha[uu] + v*D[alpha[uu],uu]/
          Sqrt[D[alpha[uu],uu].D[alpha[uu],uu]] /. uu->u

Begin["`Private`"]

kappa[ast3d[n_,a_,b_]][t_]:=
((-2 + n)*n*Cos[t]^2*Sin[t]^2*
    (35*a^2*Cos[t]^(2*n) - 
       56*a^2*Cos[t]^(2*n)*Cos[2*t] + 
       28*a^2*Cos[t]^(2*n)*Cos[4*t] - 
       8*a^2*Cos[t]^(2*n)*Cos[6*t] + 
       a^2*Cos[t]^(2*n)*Cos[8*t] + 35*b^2*Sin[t]^(2*n) + 
       8*a^2*b^2*n^2*Cos[t]^(2*n)*Sin[t]^(2*n) + 
       56*b^2*Cos[2*t]*Sin[t]^(2*n) + 
       28*b^2*Cos[4*t]*Sin[t]^(2*n) + 
       8*b^2*Cos[6*t]*Sin[t]^(2*n) + 
       b^2*Cos[8*t]*Sin[t]^(2*n))^(1/2))/
  (2^(3/2)*(16*Cos[t]^4*Sin[t]^4 + 
       a^2*n^2*Cos[t]^(2*n)*Sin[t]^4 + 
       b^2*n^2*Cos[t]^4*Sin[t]^(2*n))^(3/2))

tau[ast3d[n_,a_,b_]][t_]:=
(32*a*b*(4 - n)*Cos[t]^(2 + n)*Sin[t]^(2 + n))/
  (35*a^2*Cos[t]^(2*n) - 56*a^2*Cos[t]^(2*n)*Cos[2*t] + 
    28*a^2*Cos[t]^(2*n)*Cos[4*t] - 
    8*a^2*Cos[t]^(2*n)*Cos[6*t] + 
    a^2*Cos[t]^(2*n)*Cos[8*t] + 35*b^2*Sin[t]^(2*n) + 
    8*a^2*b^2*n^2*Cos[t]^(2*n)*Sin[t]^(2*n) + 
    56*b^2*Cos[2*t]*Sin[t]^(2*n) + 
    28*b^2*Cos[4*t]*Sin[t]^(2*n) + 
    8*b^2*Cos[6*t]*Sin[t]^(2*n) + 
    b^2*Cos[8*t]*Sin[t]^(2*n))

tandev[ast3d[n_,a_,b_]][u_,v_]:=
{a*Cos[u]^n*(1 - (n*v*Tan[u])/
      Sqrt[a^2*n^2*Cos[u]^(2*(-1 + n))*Sin[u]^2 + 
        b^2*n^2*Cos[u]^2*Sin[u]^(2*(-1 + n)) + 
        4*Sin[2*u]^2]), 
  b*Sin[u]^n*(1 + (n*v*Cot[u])/
      Sqrt[a^2*n^2*Cos[u]^(2*(-1 + n))*Sin[u]^2 + 
        b^2*n^2*Cos[u]^2*Sin[u]^(2*(-1 + n)) + 
        4*Sin[2*u]^2]), 
  Cos[2*u] - (2*v*Sin[2*u])/
    Sqrt[a^2*n^2*Cos[u]^(2*(-1 + n))*Sin[u]^2 + 
      b^2*n^2*Cos[u]^2*Sin[u]^(2*(-1 + n)) + 4*Sin[2*u]^2]}

(*
kappa[baseballseam[[a_,b_,c_]][t_]
tau[baseballseam[[a_,b_,c_]][t_]

kappa[besselcurve[a_,b_,c_]][t_]:=

tau[besselcurve[a_,b_,c_]][t_]:=
*)

kappa[bicylinder[a_,b_,pm_]][t_]:=     (* /. pm^2->1*)
(Sqrt[2]*(a^2 - 2*b^2 - a^2*Cos[2*t])^(3/2)*
    (16*a^4*b^2 - 28*a^2*b^4 + 32*b^6 - 
       18*a^4*b^2*Cos[2*t] + 48*a^2*b^4*Cos[2*t] + 
       12*a^2*b^4*Cos[4*t] + 2*a^4*b^2*Cos[6*t])^(1/2))/
  (a*(3*a^2 - 8*b^2 - 4*a^2*Cos[2*t] + a^2*Cos[4*t])^
     (3/2)*(b^2 - a^2*Sin[t]^2)^(3/2))

tau[bicylinder[a_,b_,pm_]][t_]:=     (* /. pm^2->1*)
(3*2^(5/2)*(a^2 - b^2)*pm*
    (-a^2 + 2*b^2 + a^2*Cos[2*t])^(1/2)*Sin[2*t])/
  (-8*a^4 + 14*a^2*b^2 - 16*b^4 + 9*a^4*Cos[2*t] - 
    24*a^2*b^2*Cos[2*t] - 6*a^2*b^2*Cos[4*t] - 
    a^4*Cos[6*t])

tandev[bicylinder[a_,b_,pm_]][u_,v_]:=
{a*(Cos[u] - v*Sin[u]/Sqrt[a^2*(Sin[u]^2 + 
    Cos[u]^2*(1 + a^2*pm^2*Sin[u]^2/(b^2 - a^2*Sin[u]^2)))]),
a*(Sin[u] + v*Cos[u]/Sqrt[a^2*(Sin[u]^2 + 
    Cos[u]^2*(1 + a^2*pm^2*Sin[u]^2/(b^2 - a^2*Sin[u]^2)))]),
  pm*(b^2 - a^2*Sin[u]^2 - a^2*v*Cos[u]*Sin[u]/
      Sqrt[a^2*(Sin[u]^2 + Cos[u]^2*(1 + 
               a^2*pm^2*Sin[u]^2/(b^2 - a^2*Sin[u]^2)))])/
     Sqrt[b^2 - a^2*Sin[u]^2]}

kappa[biquadratic[a_,k_]][t_]:=
Sqrt[k^2*JacobiCN[t, k]^4 + JacobiDN[t, k]^4*JacobiSN[t, k]^2*
     (JacobiDN[t, k]^2 + k^2*JacobiSN[t, k]^2) + 
    JacobiCN[t, k]^2*(JacobiDN[t, k]^6 - 
       k^2*JacobiDN[t, k]^4*JacobiSN[t, k]^2 - 
       2*k^3*JacobiDN[t, k]^2*JacobiSN[t, k]^4)]/
  (a*(JacobiDN[t, k]^2*JacobiSN[t, k]^2 + 
       JacobiCN[t, k]^2*(JacobiDN[t, k]^2 + k^2*JacobiSN[t, k]^2))^(3/2))

tau[biquadratic[a_,k_]][t_]:=
(3*k*JacobiCN[t, k]*JacobiDN[t, k]*(k*JacobiCN[t, k]^2 - JacobiDN[t, k]^2)*
    JacobiSN[t, k])/
  (a*(k^2*JacobiCN[t, k]^4 + JacobiDN[t, k]^4*JacobiSN[t, k]^2*
       (JacobiDN[t, k]^2 + k^2*JacobiSN[t, k]^2) + 
      JacobiCN[t, k]^2*(JacobiDN[t, k]^6 - 
         k^2*JacobiDN[t, k]^4*JacobiSN[t, k]^2 - 
         2*k^3*JacobiDN[t, k]^2*JacobiSN[t, k]^4)))

kappa[clelia[a_][m_,n_]][t_]:=
Sqrt[-((a^4*m^4*(192 - 480*m^2 + 420*m^4 - 126*m^6 + 
          640*n^2 - 1312*m^2*n^2 + 1040*m^4*n^2 - 
          294*m^6*n^2 + 1216*n^4 - 928*m^2*n^4 + 
          332*m^4*n^4 + 160*m^2*n^6 + 
          2*(21*m^6*(5 + 11*n^2) + 
             128*(-1 - 2*n^2 + 3*n^4) - 
             16*m^4*(21 + 47*n^2 + 10*n^4) + 
             8*m^2*(45 + 101*n^2 + 31*n^4 + 15*n^6))*
           Cos[2*n*t] - 
          8*(-8 + 36*m^2 - 42*m^4 + 15*m^6 + 16*n^2 + 
             28*m^2*n^2 - 64*m^4*n^2 + 27*m^6*n^2 - 
             8*n^4 - 52*m^2*n^4 + 10*m^4*n^4 - 12*m^2*n^6)
            *Cos[4*n*t] + 48*m^2*Cos[6*n*t] - 
          96*m^4*Cos[6*n*t] + 45*m^6*Cos[6*n*t] - 
          80*m^2*n^2*Cos[6*n*t] - 32*m^4*n^2*Cos[6*n*t] + 
          51*m^6*n^2*Cos[6*n*t] + 16*m^2*n^4*Cos[6*n*t] + 
          64*m^4*n^4*Cos[6*n*t] + 16*m^2*n^6*Cos[6*n*t] + 
          12*m^4*Cos[8*n*t] - 10*m^6*Cos[8*n*t] - 
          16*m^4*n^2*Cos[8*n*t] - 2*m^6*n^2*Cos[8*n*t] + 
          4*m^4*n^4*Cos[8*n*t] + m^6*Cos[10*n*t] - 
          m^6*n^2*Cos[10*n*t]))/(-1 + m^2*Sin[n*t]^2)^3)]/
  ((a^2*m^2*(-4 + 3*m^2 - 4*n^2 - 
         4*(-1 + m^2 + n^2)*Cos[2*n*t] + m^2*Cos[4*n*t]))/
     (-1 + m^2*Sin[n*t]^2))^(3/2)

tau[clelia[a_][m_,n_]][t_]:=
(-4*n*Sqrt[4 - 2*m^2 + 2*m^2*Cos[2*n*t]]*
    (16 - 24*m^2 + 10*m^4 + 96*n^2 - 112*m^2*n^2 + 
      62*m^4*n^2 - 112*n^4 + 136*m^2*n^4 + 
      (-16*(-1 + n^2)^2 - 3*m^4*(5 + 27*n^2) + 
         16*m^2*(2 + 7*n^2 + 3*n^4))*Cos[2*n*t] + 
      2*m^2*(-4 + 3*m^2 + 9*m^2*n^2 + 4*n^4)*Cos[4*n*t] - 
      m^4*Cos[6*n*t] + m^4*n^2*Cos[6*n*t])*Sin[2*n*t])/
  (a*(192 - 480*m^2 + 420*m^4 - 126*m^6 + 640*n^2 - 
      1312*m^2*n^2 + 1040*m^4*n^2 - 294*m^6*n^2 + 
      1216*n^4 - 928*m^2*n^4 + 332*m^4*n^4 + 
      160*m^2*n^6 + 2*(21*m^6*(5 + 11*n^2) + 
         128*(-1 - 2*n^2 + 3*n^4) - 
         16*m^4*(21 + 47*n^2 + 10*n^4) + 
         8*m^2*(45 + 101*n^2 + 31*n^4 + 15*n^6))*
       Cos[2*n*t] - 8*(-8 + 36*m^2 - 42*m^4 + 15*m^6 + 
         16*n^2 + 28*m^2*n^2 - 64*m^4*n^2 + 27*m^6*n^2 - 
         8*n^4 - 52*m^2*n^4 + 10*m^4*n^4 - 12*m^2*n^6)*
       Cos[4*n*t] + 48*m^2*Cos[6*n*t] - 
      96*m^4*Cos[6*n*t] + 45*m^6*Cos[6*n*t] - 
      80*m^2*n^2*Cos[6*n*t] - 32*m^4*n^2*Cos[6*n*t] + 
      51*m^6*n^2*Cos[6*n*t] + 16*m^2*n^4*Cos[6*n*t] + 
      64*m^4*n^4*Cos[6*n*t] + 16*m^2*n^6*Cos[6*n*t] + 
      12*m^4*Cos[8*n*t] - 10*m^6*Cos[8*n*t] - 
      16*m^4*n^2*Cos[8*n*t] - 2*m^6*n^2*Cos[8*n*t] + 
      4*m^4*n^4*Cos[8*n*t] + m^6*Cos[10*n*t] - 
      m^6*n^2*Cos[10*n*t]))

kappa[conicalhelix[a_,b_,c_,d_]][t_]:=
(d*Sqrt[8*a^2*b^2 + 4*a^2*c^2 + 4*b^2*c^2 + 
      8*a^2*b^2*d^2*t^2 + a^2*c^2*d^2*t^2 + 
      b^2*c^2*d^2*t^2 + 2*a^2*b^2*d^4*t^4 + 
      (a^2 - b^2)*c^2*(-4 + d^2*t^2)*Cos[2*d*t] + 
      4*(a^2 - b^2)*c^2*d*t*Sin[2*d*t]])/
  (Sqrt[2]*(c^2 + (a^2 + b^2*d^2*t^2)*Cos[d*t]^2 + 
       (b^2 + a^2*d^2*t^2)*Sin[d*t]^2 - 
       a^2*d*t*Sin[2*d*t] + b^2*d*t*Sin[2*d*t])^(3/2))

tau[conicalhelix[a_,b_,c_,d_]][t_]:=
(2*a*b*c*d*(6 + d^2*t^2))/
  (8*a^2*b^2 + 4*a^2*c^2 + 4*b^2*c^2 + 
    8*a^2*b^2*d^2*t^2 + a^2*c^2*d^2*t^2 + 
    b^2*c^2*d^2*t^2 + 2*a^2*b^2*d^4*t^4 + 
    (a^2 - b^2)*c^2*(-4 + d^2*t^2)*Cos[2*d*t] + 
    4*(a^2 - b^2)*c^2*d*t*Sin[2*d*t])

tandev[conicalhelix[a_,b_,c_,d_]][u_,v_]:=
{a*u*Cos[d*u] + (v*(a*Cos[d*u] - a*d*u*Sin[d*u]))/
    Sqrt[c^2 + (b*d*u*Cos[d*u] + b*Sin[d*u])^2 + 
      (a*Cos[d*u] - a*d*u*Sin[d*u])^2], 
  b*u*Sin[d*u] + (v*(b*d*u*Cos[d*u] + b*Sin[d*u]))/
    Sqrt[c^2 + (b*d*u*Cos[d*u] + b*Sin[d*u])^2 + 
      (a*Cos[d*u] - a*d*u*Sin[d*u])^2], 
  c*u + (c*v)/
    Sqrt[c^2 + (b*d*u*Cos[d*u] + b*Sin[d*u])^2 + 
      (a*Cos[d*u] - a*d*u*Sin[d*u])^2]}

kappa[cubicalellipse[a_,b_,c_,d_,e_,f_,pm_]][t_]:=
2*(1 + pm*t^2)^3*Sqrt[a^2*c^2 + c^2*d^2 - 2*b*c*d*e + 
       a^2*e^2 + b^2*e^2 - 6*b*c*d*f*t + 6*a^2*e*f*t + 
       6*b^2*e*f*t - 6*c^2*d*f*t^2 + 6*b*c*e*f*t^2 + 
       9*a^2*f^2*t^2 + 9*b^2*f^2*t^2 + 18*b*c*f^2*t^3 + 
       2*b*c*d*f*pm*t^3 - 2*a^2*e*f*pm*t^3 - 
       2*b^2*e*f*pm*t^3 + 9*c^2*f^2*t^4 - 
       6*a^2*f^2*pm*t^4 - 6*b^2*f^2*pm*t^4 - 
       6*b*c*f^2*pm*t^5 + a^2*f^2*t^6 + b^2*f^2*t^6]/
    (a^2 + b^2 + d^2 + 4*b*c*t + 4*d*e*t + 4*c^2*t^2 + 
     4*e^2*t^2 + 6*d*f*t^2 - 2*a^2*pm*t^2 - 
     2*b^2*pm*t^2 - 2*d^2*pm*t^2 + 12*e*f*t^3 - 
     4*b*c*pm*t^3 - 4*d*e*pm*t^3 + a^2*t^4 + b^2*t^4 + 
     d^2*t^4 + 9*f^2*t^4 - 4*d*f*pm*t^4 + 4*e*f*pm*t^5 - 
     2*d*f*t^6 + 6*f^2*pm*t^6 + f^2*t^8)^(3/2)

tau[cubicalellipse[a_,b_,c_,d_,e_,f_,pm_]][t_]:=
(3*a*c*f*(1 + pm*t^2)^2)/
  (a^2*c^2 + c^2*d^2 - 2*b*c*d*e + a^2*e^2 + b^2*e^2 - 
    6*b*c*d*f*t + 6*a^2*e*f*t + 6*b^2*e*f*t - 
    6*c^2*d*f*t^2 + 6*b*c*e*f*t^2 + 9*a^2*f^2*t^2 + 
    9*b^2*f^2*t^2 + 18*b*c*f^2*t^3 + 2*b*c*d*f*pm*t^3 - 
    2*a^2*e*f*pm*t^3 - 2*b^2*e*f*pm*t^3 + 9*c^2*f^2*t^4 - 
    6*a^2*f^2*pm*t^4 - 6*b^2*f^2*pm*t^4 - 
    6*b*c*f^2*pm*t^5 + a^2*f^2*t^6 + b^2*f^2*t^6)

tandev[cubicalellipse[a_,b_,c_,d_,e_,f_,pm_]][u_,v_]:=
{(a*(u + pm*u^3 - ((-1 + pm*u^2)*v)/
        Sqrt[(a^2*(-1 + pm*u^2)^2 + 
            (b + 2*c*u - b*pm*u^2)^2 + 
            (d - d*pm*u^2 + u*(2*e + f*u*(3 + pm*u^2)))^2)
           /(1 + pm*u^2)^4]))/(1 + pm*u^2)^2, 
  (u*(b + c*u)*(1 + pm*u^2) - 
     ((-2*c*u + b*(-1 + pm*u^2))*v)/
      Sqrt[(a^2*(-1 + pm*u^2)^2 + 
          (b + 2*c*u - b*pm*u^2)^2 + 
          (d - d*pm*u^2 + u*(2*e + f*u*(3 + pm*u^2)))^2)/
        (1 + pm*u^2)^4])/(1 + pm*u^2)^2, 
  (u*(1 + pm*u^2)*(d + u*(e + f*u)) + 
     ((d - d*pm*u^2 + u*(2*e + f*u*(3 + pm*u^2)))*v)/
      Sqrt[(a^2*(-1 + pm*u^2)^2 + 
          (b + 2*c*u - b*pm*u^2)^2 + 
          (d - d*pm*u^2 + u*(2*e + f*u*(3 + pm*u^2)))^2)/
        (1 + pm*u^2)^4])/(1 + pm*u^2)^2}

kappa[helix[a_,b_,c_]][t_]:=
Sqrt[2*a^2*b^2 + a^2*c^2 + b^2*c^2 + a^2*c^2*Cos[2*t] - 
     b^2*c^2*Cos[2*t]]/
  (Sqrt[2]*(c^2 + b^2*Cos[t]^2 + a^2*Sin[t]^2)^(3/2))

tau[helix[a_,b_,c_]][t_]:=
2*a*b*c/(2*a^2*b^2 + a^2*c^2 + b^2*c^2 + 
    a^2*c^2*Cos[2*t] - b^2*c^2*Cos[2*t])

tandev[helix[a_,b_,c_]][u_,v_]:=
{a*Cos[u] - a*v*Sin[u]/Sqrt[c^2 + b^2*Cos[u]^2 + a^2*Sin[u]^2], 
  b*Sin[u] + b*v*Cos[u]/Sqrt[c^2 + b^2*Cos[u]^2 + a^2*Sin[u]^2], 
  c*u + c*v/Sqrt[c^2 + b^2*Cos[u]^2 + a^2*Sin[u]^2]}

kappa[horopter[a_,b_]][t_]:=
(4*a*b^2*Sqrt[4*a^2*b^2 + (1 + b^2*t^2)^3])/
  (4*a^2*b^2 + (1 + b^2*t^2)^2)^(3/2)

tau[horopter[a_,b_]][t_]:=
3*b*(1 + b^2*t^2)^2/(4*a^2*b^2 + (1 + b^2*t^2)^3)

tandev[horopter[a_,b_]][u_,v_]:=
{(a*(2 - (4*b^2*u*v)/Sqrt[4*a^2*b^2 + (1 + b^2*u^2)^2]))/(1 + b^2*u^2),
(2*a*b*(u + b^2*u^3 + (v - b^4*u^4*v)/Sqrt[4*a^2*b^2 + (1 + b^2*u^2)^2]))/
   (1 + b^2*u^2)^2,
u + ((1 + b^2*u^2)*v)/Sqrt[4*a^2*b^2 + (1 + b^2*u^2)^2]}

kappa[line3d[a1_,b1_,c1_,a2_,b2_,c2_]][t_]:=
0


tau[line3d[a1_,b1_,c1_,a2_,b2_,c2_]][t_]:=
0

(*
kappa[pseudospheregeodesic[a_,c_,u0_,pm_]][t_]:=

tau[pseudospheregeodesic[a_,c_,u0_,pm_]][t_]:=

tandev[pseudospheregeodesic[a_,c_,u0_,pm_]][u_,v_]:=
*)

kappa[pseudosphericalloxodrome[a_,b_]][t_]:=
(Sqrt[8 + 4*b^2 - b^4 - 4*b^2*Cosh[2*t] + b^4*Cosh[4*t]]*
    Csch[t])/(2*Sqrt[2]*a*(1 + b^2))

tau[pseudosphericalloxodrome[a_,b_]][t_]:=
(b^3*Cosh[t]*(-4*(2 + b^2)*Cosh[2*t] + 
      b^2*(3 + Cosh[4*t]))*Coth[t])/
  (a*(1 + b^2)*(8 + 4*b^2 - b^4 - 4*b^2*Cosh[2*t] + 
      b^4*Cosh[4*t]))

tandev[pseudosphericalloxodrome[a_,b_]][u_,v_]:=
{a*Sech[u]*(Cos[b*Cosh[u]] - 
     (v*(Cos[b*Cosh[u]] + b*Cosh[u]*Sin[b*Cosh[u]]))/
      (a*Sqrt[1 + b^2])), 
  a*Sech[u]*((v*(b*Cos[b*Cosh[u]]*Cosh[u] - 
          Sin[b*Cosh[u]]))/(a*Sqrt[1 + b^2]) + 
     Sin[b*Cosh[u]]), a*
   (u - Tanh[u] + (v*Tanh[u])/(a*Sqrt[1 + b^2]))}

kappa[sphericalcardioid[a_]][t_]:=
Sqrt[3 - 4*Cos[t] + Cos[2*t]]/(3*a*(1 - Cos[t])^(3/2))

tau[sphericalcardioid[a_]][t_]:=
-Csc[t/2]/(6*Sqrt[2]*a)

tandev[sphericalcardioid[a_]][u_,v_]:=
{2*a*Cos[u] + (2*Sqrt[2]*v*Cos[(3*u)/2])/3 - a*Cos[2*u], 
  2*a*Sin[u] + (2*Sqrt[2]*v*Sin[(3*u)/2])/3 - 
   a*Sin[2*u], -v/3 + 2*Sqrt[2]*a*Cos[u/2]}

kappa[sphericalhelix[a_]][t_]:=
Csc[(a*t)/(2*b)]/(a + 2*b)

tau[sphericalhelix[a_]][t_]:=
-Csc[t/2]/(6*Sqrt[2]*a)

tandev[sphericalhelix[a_]][u_,v_]:=
{(a + b)*Cos[u] - b*Cos[((a + b)*u)/b] + 
   (2*Sqrt[b]*Sqrt[a + b]*v*Cos[u + (a*u)/(2*b)])/
    (a + 2*b), (a + b)*Sin[u] - b*Sin[((a + b)*u)/b] + 
   (2*Sqrt[b]*Sqrt[a + b]*v*Sin[u + (a*u)/(2*b)])/
    (a + 2*b), -((a*v)/(a + 2*b)) + 
   2*Sqrt[b]*Sqrt[a + b]*Cos[(a*u)/(2*b)]}

kappa[sphericalloxodrome[a_,b_,pm_]][t_]:=
((1 + b^2)*(1 + t^2)^3*
    (1 + 2*t^2 + 4*b^2*t^2 + t^4)^(1/2))/
  (2*a*t*((1 + b^2)*(1 + t^2)^2)^(3/2))

tau[sphericalloxodrome[a_,b_,pm_]][t_]:=
-((b*(1 + t^2)^2)/(a*(1 + 2*t^2 + 4*b^2*t^2 + t^4)))

tandev[sphericalloxodrome[a_,b_,pm_]][u_,v_]:=
{(-2*a*b*E^(c*u)*Cos[u])/(1 + b^2*E^(2*c*u)) + 
   Sqrt[1 + c^2]*((-2*c*v*Cos[u])/
       ((1 + c^2)*(1 + b^2*E^(2*c*u))) + 
      (v*(c*Cos[u] + Sin[u]))/(1 + c^2)), 
  (2*a*b*E^(c*u)*Sin[u])/(1 + b^2*E^(2*c*u)) + 
   Sqrt[1 + c^2]*((2*c*v*Sin[u])/
       ((1 + c^2)*(1 + b^2*E^(2*c*u))) - 
      (v*(-Cos[u] + c*Sin[u]))/(1 + c^2)), 
  a - (2*a)/(1 + b^2*E^(2*c*u)) + 
   (2*b*c*E^(c*u)*v)/(Sqrt[1 + c^2]*(1 + b^2*E^(2*c*u)))}

kappa[sphericalnephroid[a_]][t_]:=
Csc[t]/(4*a)

tau[sphericalnephroid[a_]][t_]:=
-Csc[t]/(4*Sqrt[3]*a)

tandev[sphericalnephroid[a_]][u_,v_]:=
{3*a*Cos[u] + (Sqrt[3]*v*Cos[2*u])/2 - a*Cos[3*u], 
  Sqrt[3]*v*Cos[u]*Sin[u] + 4*a*Sin[u]^3, 
  -v/2 + 2*Sqrt[3]*a*Cos[u]}

kappa[sphericalspiral[a_][m_,n_]][t_]:=
(3*m^6 + 13*m^4*n^2 + 28*m^2*n^4 + 8*n^6 + 
     4*m^6*Cos[2*n*t] + 12*m^4*n^2*Cos[2*n*t] - 
     4*m^2*n^4*Cos[2*n*t] + m^6*Cos[4*n*t] - 
     m^4*n^2*Cos[4*n*t])^(1/2)/
  (a*(m^2 + 2*n^2 + m^2*Cos[2*n*t])^(3/2))

tau[sphericalspiral[a_][m_,n_]][t_]:=
(4*m*n*Cos[n*t]*(m^4 + 7*m^2*n^2 + 4*n^4 + 
      m^4*Cos[2*n*t] - m^2*n^2*Cos[2*n*t]))/
  (a*(3*m^6 + 13*m^4*n^2 + 28*m^2*n^4 + 8*n^6 + 
      4*m^6*Cos[2*n*t] + 12*m^4*n^2*Cos[2*n*t] - 
      4*m^2*n^4*Cos[2*n*t] + m^6*Cos[4*n*t] - 
      m^4*n^2*Cos[4*n*t]))

tandev[sphericalspiral[a_][m_,n_]][u_,v_]:=
{a*Cos[m*u]*Cos[n*u] +
Sqrt[2]*v*(-a*m*Cos[n*u]*Sin[m*u] - a*n*Cos[m*u]*Sin[n*u])/
(a*Sqrt[m^2 + 2*n^2 + m^2*Cos[2*n*u]]), 
 a*Cos[n*u]*Sin[m*u] + 
   Sqrt[2]*v*(a*m*Cos[m*u]*Cos[n*u] - a*n*Sin[m*u]*Sin[n*u])/
    (a*Sqrt[m^2 + 2*n^2 + m^2*Cos[2*n*u]]), 
  Sqrt[2]*n*v*Cos[n*u]/
    Sqrt[m^2 + 2*n^2 + m^2*Cos[2*n*u]] + a*Sin[n*u]}


kappa[torusknot[a_,b_,c_][p_,q_]][t_]:=
Sqrt[8*a^4*p^6 + 24*a^2*b^2*p^6 + 3*b^4*p^6 + 
     40*a^2*b^2*p^4*q^2 + 10*b^4*p^4*q^2 + 
     4*a^2*c^2*p^4*q^2 + 3*b^2*c^2*p^4*q^2 + 
     4*a^2*b^2*p^2*q^4 + 19*b^4*p^2*q^4 + 
     4*a^2*c^2*p^2*q^4 + 9*b^2*c^2*p^2*q^4 + 
     8*b^2*c^2*q^6 + 32*a^3*b*p^6*Cos[q*t] + 
     24*a*b^3*p^6*Cos[q*t] + 16*a^3*b*p^4*q^2*Cos[q*t] + 
     52*a*b^3*p^4*q^2*Cos[q*t] + 
     12*a*b*c^2*p^4*q^2*Cos[q*t] + 
     20*a*b^3*p^2*q^4*Cos[q*t] + 
     12*a*b*c^2*p^2*q^4*Cos[q*t] + 
     24*a^2*b^2*p^6*Cos[2*q*t] + 4*b^4*p^6*Cos[2*q*t] + 
     8*a^2*b^2*p^4*q^2*Cos[2*q*t] + 
     8*b^4*p^4*q^2*Cos[2*q*t] + 
     4*a^2*c^2*p^4*q^2*Cos[2*q*t] + 
     4*b^2*c^2*p^4*q^2*Cos[2*q*t] + 
     4*a^2*b^2*p^2*q^4*Cos[2*q*t] - 
     12*b^4*p^2*q^4*Cos[2*q*t] - 
     4*a^2*c^2*p^2*q^4*Cos[2*q*t] + 
     8*b^2*c^2*p^2*q^4*Cos[2*q*t] + 
     8*a*b^3*p^6*Cos[3*q*t] - 
     4*a*b^3*p^4*q^2*Cos[3*q*t] + 
     4*a*b*c^2*p^4*q^2*Cos[3*q*t] - 
     4*a*b^3*p^2*q^4*Cos[3*q*t] + 
     4*a*b*c^2*p^2*q^4*Cos[3*q*t] + b^4*p^6*Cos[4*q*t] - 
     2*b^4*p^4*q^2*Cos[4*q*t] + 
     b^2*c^2*p^4*q^2*Cos[4*q*t] + 
     b^4*p^2*q^4*Cos[4*q*t] - b^2*c^2*p^2*q^4*Cos[4*q*t]]/
     (2*a^2*p^2 + b^2*p^2 + b^2*q^2 + c^2*q^2 + 
     4*a*b*p^2*Cos[q*t] + b^2*p^2*Cos[2*q*t] - 
     b^2*q^2*Cos[2*q*t] + c^2*q^2*Cos[2*q*t])^(3/2)


tau[torusknot[a_,b_,c_][p_,q_]][t_]:=
(2*c*p*q*(4*a*b*p^4 - 4*a*b*q^4 + 4*a^2*p^4*Cos[q*t] + 
      3*b^2*p^4*Cos[q*t] - 4*a^2*p^2*q^2*Cos[q*t] + 
      13*b^2*p^2*q^2*Cos[q*t] + 8*b^2*q^4*Cos[q*t] + 
      4*a*b*p^4*Cos[2*q*t] + 8*a*b*p^2*q^2*Cos[2*q*t] + 
      b^2*p^4*Cos[3*q*t] - b^2*p^2*q^2*Cos[3*q*t]))/
  (8*a^4*p^6 + 24*a^2*b^2*p^6 + 3*b^4*p^6 + 
    40*a^2*b^2*p^4*q^2 + 10*b^4*p^4*q^2 + 
    4*a^2*c^2*p^4*q^2 + 3*b^2*c^2*p^4*q^2 + 
    4*a^2*b^2*p^2*q^4 + 19*b^4*p^2*q^4 + 
    4*a^2*c^2*p^2*q^4 + 9*b^2*c^2*p^2*q^4 + 
    8*b^2*c^2*q^6 + 32*a^3*b*p^6*Cos[q*t] + 
    24*a*b^3*p^6*Cos[q*t] + 16*a^3*b*p^4*q^2*Cos[q*t] + 
    52*a*b^3*p^4*q^2*Cos[q*t] + 
    12*a*b*c^2*p^4*q^2*Cos[q*t] + 
    20*a*b^3*p^2*q^4*Cos[q*t] + 
    12*a*b*c^2*p^2*q^4*Cos[q*t] + 
    24*a^2*b^2*p^6*Cos[2*q*t] + 4*b^4*p^6*Cos[2*q*t] + 
    8*a^2*b^2*p^4*q^2*Cos[2*q*t] + 
    8*b^4*p^4*q^2*Cos[2*q*t] + 
    4*a^2*c^2*p^4*q^2*Cos[2*q*t] + 
    4*b^2*c^2*p^4*q^2*Cos[2*q*t] + 
    4*a^2*b^2*p^2*q^4*Cos[2*q*t] - 
    12*b^4*p^2*q^4*Cos[2*q*t] - 
    4*a^2*c^2*p^2*q^4*Cos[2*q*t] + 
    8*b^2*c^2*p^2*q^4*Cos[2*q*t] + 
    8*a*b^3*p^6*Cos[3*q*t] - 4*a*b^3*p^4*q^2*Cos[3*q*t] + 
    4*a*b*c^2*p^4*q^2*Cos[3*q*t] - 
    4*a*b^3*p^2*q^4*Cos[3*q*t] + 
    4*a*b*c^2*p^2*q^4*Cos[3*q*t] + b^4*p^6*Cos[4*q*t] - 
    2*b^4*p^4*q^2*Cos[4*q*t] + 
    b^2*c^2*p^4*q^2*Cos[4*q*t] + b^4*p^2*q^4*Cos[4*q*t] - 
    b^2*c^2*p^2*q^4*Cos[4*q*t])

kappa[twicubic][t_]:=
Sqrt[4 + 36*t^2 + 36*t^4]/(1 + 4*t^2 + 9*t^4)^(3/2)

tau[twicubic][t_]:=
3/(1 + 9*t^2 + 9*t^4)

tandev[twicubic][u_,v_]:=
          {u + v/Sqrt[1 + 4*u^2 + 9*u^4], 
           u^2 + 2*u*v/Sqrt[1 + 4*u^2 + 9*u^4], 
           u^3 + 3*u^2*v/Sqrt[1 + 4*u^2 + 9*u^4]}

kappa[veryflat[a_]][t_]:=0

tau[veryflat[a_]][t_]:=0

kappa[viviani[a_]][t_]:=
Sqrt[13 + 3*Cos[t]]/(a*(3 + Cos[t])^(3/2))

tau[viviani[a_]][t_]:=
(6*Cos[t/2])/(a*(13 + 3*Cos[t]))

tangent[viviani[a_]][t_]:=
{-((Sqrt[2]*Sin[t])/Sqrt[3 + Cos[t]]), 
  (Sqrt[2]*Cos[t])/Sqrt[3 + Cos[t]], 
  (Sqrt[2]*Cos[t/2])/Sqrt[3 + Cos[t]]}

normal[viviani[a_]][t_]:=
{(-4*Cos[t/2]^4)/
    (Sqrt[3 + Cos[t]]*Sqrt[13 + 3*Cos[t]]) - 
   (4*Cos[t])/(Sqrt[3 + Cos[t]]*Sqrt[13 + 3*Cos[t]]), 
  (-4*Sin[t])/(Sqrt[3 + Cos[t]]*Sqrt[13 + 3*Cos[t]]) - 
   (Cos[t/2]*(3*Sin[t/2] + Sin[(3*t)/2]))/
    (Sqrt[3 + Cos[t]]*Sqrt[13 + 3*Cos[t]]), 
  (-4*Cos[t/2]^3*Sin[t])/
    (Sqrt[3 + Cos[t]]*Sqrt[13 + 3*Cos[t]]) + 
   (Cos[t]*(3*Sin[t/2] + Sin[(3*t)/2]))/
    (Sqrt[3 + Cos[t]]*Sqrt[13 + 3*Cos[t]])}

binormal[viviani[a_]][t_]:=
{(3*Sin[t/2] + Sin[(3*t)/2])/
   (Sqrt[2]*Sqrt[13 + 3*Cos[t]]), 
  (-2*Sqrt[2]*Cos[t/2]^3)/Sqrt[13 + 3*Cos[t]], 
  (2*Sqrt[2])/Sqrt[13 + 3*Cos[t]]}

tandev[viviani[a_]][u_,v_]:=
         {a*(1 + Cos[u]) - a*v*Sin[u],
	  a*v*Cos[u] + a*Sin[u], 
          a*v*Cos[u/2] + 2*a*Sin[u/2]}

mcurvature[tandev[viviani[a_]]][u_,v_]:=
-3*Sqrt[2]*a^5*v^2*Cos[u/2]*(3 + Cos[u])/
  (a^4*v^2*(13 + 3*Cos[u]))^(3/2)

End[]

EndPackage[]
